# -*- coding: utf-8 -*-

import unittest
from wosedon.reranks import LemmaRankingSelecter
from wosedon.basegraph import BaseGraph
from wosedon.utils.config import Config



class TestLemmaRankingSelecter(unittest.TestCase):
  
  def setUp(self):
    class Synset:
      pass
    
    cfg = Config()
    cfg.parse('tests/#Data/wosedon_cosine.ini')
    cfg.resources().set_tagset('nkjp')
    cfg._context = 'Document'
    ctx = cfg.context('tests/#Data/little_ccl.xml', 'tests/#Data/little_ccl.xml')
    
    g = BaseGraph()
    g.init_graph()
    g.create_node_attribute('synset', 'object')
    
    self.nodes = []
    for i in xrange(10):
      self.nodes.append(g.add_node(i, [('synset', Synset())]))
    
    self.tokens = ctx.next().tokens()
    self.graph = g
    self.rerank = LemmaRankingSelecter()
    self.options = cfg.rerank_options()
  
  
  def test_it(self):
    class LU:
      def __init__(self, lemma, variant):
        self.lemma = lemma
        self.variant = variant
        
    self.nodes[0].synset.lu_set = {LU('podatek', 5)}
    self.nodes[1].synset.lu_set = {LU('podatek', 4)}
    self.nodes[2].synset.lu_set = {LU('podatek', 1)}
    self.nodes[3].synset.lu_set = {LU('w', 3)}
    self.nodes[4].synset.lu_set = {LU('unia', 2)}
    self.nodes[5].synset.lu_set = {LU('unia', 1)}
    self.nodes[6].synset.lu_set = {LU('europejski', 3)}
    self.nodes[7].synset.lu_set = {LU('można', 3)}
    self.nodes[8].synset.lu_set = {LU('można', 2)}
    self.nodes[9].synset.lu_set = {LU('można', 1)}
    
    ranking = [
      (self.tokens[0], [(self.nodes[0], 100), (self.nodes[1], 99), (self.nodes[2], 98)]),
      (self.tokens[1], [(self.nodes[3], 5)]),
      (self.tokens[2], [(self.nodes[4], 11), (self.nodes[5], 1)]),
      (self.tokens[3], [(self.nodes[6], 13)]),
      (self.tokens[4], [(self.nodes[7], 100), (self.nodes[8], 99), (self.nodes[9], 1)])
    ]
    
    new_rank = self.rerank.rerank(ranking, self.graph, self.options)
    
    self.assertEquals(len(new_rank), 5)
    for i, (tok, rank) in enumerate(new_rank):
      self.assertIs(tok, self.tokens[i])
    
    self.assertEquals(new_rank[0][1][0][0], self.nodes[2])
    self.assertEquals(new_rank[1][1][0][0], self.nodes[3])
    self.assertEquals(new_rank[2][1][0][0], self.nodes[4])
    self.assertEquals(new_rank[3][1][0][0], self.nodes[6])
    self.assertEquals(new_rank[4][1][0][0], self.nodes[8])
    






#
# <one line to give the library's name and an idea of what it does.>
# Copyright (C) 2014  <copyright holder> <email>
# 
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
# 
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# 
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
# 
#

import os
from wosedon.basegraph import BaseGraph
from graphbuilderinterface import GraphBuilderInterface

class LexicalUnitGraphBuilder(GraphBuilderInterface):
  """!
  Class for building lexical unit graph.

  The built graph is based on lexical unit graph file. The lexical unit graph is 
  unpickled from file and according to build options set in config file some nodes 
  and edges are filtered out.
  """
  def __init__(self, resources, options, str_name = 'LexicalUnitGraphBuilder'):
    """!
    @param resources - object of Resources class contain resources from config 
                       file
    @param options   - object of BuildOptions class contain build options from 
                       config file
    @param str_name  - default value same as the class name 
                       "LexicalUnitGraphBuilder"
    """
    super(LexicalUnitGraphBuilder, self).__init__(resources, options, str_name)

  def build_graph(self):
    """!
    Build lexical unit graph. 

    At the beginning the lexical unit graph is unpickled 
    from file, which path is set in config file. Next some nodes and edges are 
    filtered out according to build options, which is also set in config file. Each 
    node has an attribute named "lu" which contain lexical unit object. Lexical unit 
    object consist of the following fields: lu_id, lemma, pos, domain and variant. Whereas 
    to each edge is assigned relation identifier acquired from PLWN, which occurs 
    between linked lexical units.

    The directness of the graph and uniqueness of edges is set in the config file. 

    @return object of BaseGraph class
    """
    lu_graph_file_path = \
      self.resources().plwn_graph_file() + '_lu.xml.gz'

    if not os.path.exists(lu_graph_file_path):
      raise IOError(
        "%s file not found!" % \
          lu_graph_file_path)

    g = BaseGraph()
    g.init_graph(drctd = self.options().is_directed_graph())
    g.unpickle(lu_graph_file_path)
    g.set_directed(self.options().is_directed_graph())

    # Vertices filter.
    if self.options().accept_pos() or self.options().accept_lexicon():
      nodes_to_filter_set = set()
      for n in g.all_nodes():
        if self.options().accept_lexicon():
          lexicon = n.lu.lexicon
          if not lexicon in self.options().accept_lexicon():
            nodes_to_filter_set.add(n)

        if self.options().accept_pos():
          pos = n.lu.pos
          if not pos in self.options().accept_pos():
            nodes_to_filter_set.add(n)

      g.nodes_filter(nodes_to_filter_set)
    
    # Edges
    # Add reversed edges
    
    lst = set()
    for edge in g.all_edges():
      if edge.rel_id in self.options().get_add_reversed_edges():
        lst.add((edge.target(), edge.source(), edge.rel_id))
    for s, t, i in lst:
      ne = g.add_edge(s, t, simply=True)
      ne.rel_id = self.options().get_add_reversed_edges()[i]
    
    # Check edges duplicates
    if self.options().has_unique_edges():
      g.remove_edge_duplicates()
      
    # Edges filter
    if self.options().lu_rel_ids():
      edges_to_filter_set = set()
      for e in g.all_edges():
        if not e.rel_id in self.options().lu_rel_ids():
          edges_to_filter_set.add(e)
      g.edges_filter(edges_to_filter_set)
    
    # Type of relation
    g.create_edge_attribute("rel", "string")
    for e in g.all_edges():
      e.rel = 'l' + str(e.rel_id)

    return g



#!/usr/bin/python
# -*- coding: utf-8 -*-

# <one line to give the library's name and an idea of what it does.>
# Copyright (C) 2014  <copyright holder> <email>
# 
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
# 
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# 
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#

'''
WoSeDon - Word Sense Disambiguation tool for processing polish language.
WoSeDon based on the PageRank approach to make disambiguation, uses
many resources to build the graphs. These graphs can be merged into one
larger by many available mergers.
'''

import os, sys, time, logging, tempfile
import corpus2
import wosedon_base
from utils.config import Config
#from ranking.wsd_ranking import WSDRanking
from visualisation import Visualisation


def _make_parser(desc = ''):
  aparser = wosedon_base.make_parser(desc)
  meg = aparser.add_mutually_exclusive_group(required = True)
  meg.add_argument('-f', '--cclfile', dest = 'ccl',
                   type = str, help = 'Ccl file.')
  meg.add_argument('-i', '--interactive', dest = 'interactive',
                   action = 'store_true',
                   help = (
                     'If interactive mode is enabled, then instead of a filename '
                     'argument the user is expected to provide the Ccl-formatted '
                     'data to be parsed as WoSeDon\'s input. In this mode '
                     'processing is performed in a loop: the user can provide '
                     'several chunks of data, and they will be processed in '
                     'order. Each must however be terminated with an empty line.')),
  aparser.add_argument('-r', '--relcclfile', dest = 'relccl',
                       required = False, type = str, help = 'Relccl file')
  aparser.add_argument('-o', '--out-file', dest = 'out_file', default = '-',
                       help = (
                         'Output file. Default is set to write output '
                         'to stdout (\"-\") if set filename, then output will be ' 
                         'stored to given filename.'))
  aparser.add_argument('-b', '--batch', dest = 'batch',
                       action = 'store_true',
                       help = (
                         'If batch mode is enabled, then into -f file is '
                         'the index of files to be disambiguated. The index '
                         'contains the paths to the files: one path per one '
                         'line. New, disambiguated file has new name with '
                         'postfix like a *.wosedon.xml')),
  aparser.add_argument('-vd', '--visualisation-dir', dest = 'visualisation_dir',
                       required = False,
                       help = ('Path to directory for PageRank visualisation.'))
  return aparser


def _run_disambiguation(graph, context, wsd_alg, cfg, alfa=None, v_dir=None, 
                        output_filename=None, show_iterations=False,
                        model_result_list=None):
  
  logger = logging.getLogger(__name__)
  
  
  ret_iter_list = []
  # for given context in "c" run disambiguation method
  # to disambiguation use the built "graph" graph
  c = context.next()
  while c:
    (wsd_rank, ret_iter) = wsd_alg.run(
      c, graph, cfg.algorithm_options(), cfg.resources())
    ctx_rank = wsd_rank.get_ranking_for_context(c, graph)
    # run reranking for all rerankers
    for reranker in cfg.rerankers():
      logger.info('Reranking using %s.', str(reranker))
      ctx_rank = reranker.rerank(ctx_rank, graph, cfg.rerank_options())

    # add annotation
    if model_result_list != None:
      model_result_list.append(ctx_rank)
    else:
      wosedon_base.add_annotations_from_rank(ctx_rank, graph, alfa)

    if show_iterations:
      ret_iter_list.append(ret_iter)
    if v_dir:
      Visualisation().make_visualisation(c, graph, wsd_rank, v_dir)
    c =  context.next()
  # the context should be reseted at the end
  context.reset()
  
  if output_filename:
    # write the output filename
    wosedon_base.write_ccl_file(cfg, context.ccl_document(), output_filename)

  if show_iterations:
    for i, ret_iter in enumerate(ret_iter_list):
      looger.info("Total number of iterations (context %s):", str(i), str(ret_iter))


def main(argv = None):
  logger = logging.getLogger(__name__)
  
  p = _make_parser(__doc__)
  args = p.parse_args(argv)

  cfg = _read_config(args.config)
  builders, mergers, wsd_alg, g =_parse_cfg_object(cfg, args)

  if args.batch:
    with open(args.ccl, 'rt') as batch_file:
      for line in batch_file:
        start = time.clock()
        ccl_filename = line.strip()
        out_ccl_filename = ccl_filename + ".wosedon.xml"
        logger.info('Processing %s.', ccl_filename)

        # makes contex object based on the ccl file from ccl index file
        context = cfg.context(ccl_filename, ccl_filename)
        try:
          _run_disambiguation(
            g, 
            context, 
            wsd_alg, 
            cfg, 
            float(args.alpha) if args.alpha else None, 
            args.visualisation_dir,
            out_ccl_filename, 
            args.iterations)
          logger.info('Done in %.2f.', time.clock() - start)
        except Exception, e:
          print e
  elif args.interactive:
    tmp = tempfile.NamedTemporaryFile()
    l = sys.stdin.readline()
    while len(l) > 0:
      if len(l) > 1:
        tmp.write(l)
      else:
        tmp.seek(0)

        context = cfg.context(tmp.name, tmp.name)
        _run_disambiguation(
          g, 
          context, 
          wsd_alg, 
          cfg, 
          float(args.alpha) if args.alpha else None,
          args.visualisation_dir,
          args.out_file,
          args.iterations
        )
            
        tmp.close()
        tmp = tempfile.NamedTemporaryFile()

      l = sys.stdin.readline()

    tmp.close()  
  else:
    context = cfg.context(args.ccl, 
                          args.ccl if not args.relccl else args.relccl)
    _run_disambiguation(
      g, 
      context, 
      wsd_alg, 
      cfg, 
      float(args.alpha) if args.alpha else None,
      args.visualisation_dir,
      args.out_file,
      args.iterations
    )


def disambiguate_document(document, config, wsd_objects=None):
  """!
  Run WoSeDon disambiguation on the given document object.

  The input document is an document object of corpus2. The path to
  a configuration has to contain all sections required by WoSeDon.
  The output is a document (for future use, right now it is the same object as
  for the input) and a tuple of WSD objects that may be used for the next run
  of WSD (unless you want to use another configuration file

  @param document a document object to be disambiguated.
  @type document: corpus2.Document|corpus2.DocumentPtr
  @param config the path to a configuration file or a configuration object.
  @type config: str|wosedon.utils.config.Config
  @param wsd_objects WSD objects from a previous run. If not given they will
    be created from the scratch based on the provided configuration file.
  @type wsd_objects: tuple

  @return a disambiguated document object, a configuration object and
    WSD objects.
  @rtype: (corpus2.Document|corpus2.DocumentPtr, wosedon.utils.config.Config,
    tuple)
  """
  if isinstance(config, str):
    cfg = _read_config(config)
  else:
    cfg = config
  if wsd_objects:
    builders, mergers, wsd_alg, graph = wsd_objects
  else:
    builders, mergers, wsd_alg, graph = _parse_cfg_object(cfg)
  context = cfg.create_context(document)
  _run_disambiguation(
      graph, 
      context, 
      wsd_alg, 
      cfg
  )
  return document, cfg, (builders, mergers, wsd_alg, graph)

def _read_config(config_path):
  """! Load and return Config object. """
  return Config(config_path)

def _parse_cfg_object(cfg, args=None):
  """! Parse the config object to base objects and return them. """
  builders, mergers, wsd_alg, graphs = \
      wosedon_base.parse_cfg_to_base_objects(cfg, args)
  # FIXME: Uwaga! konfig moze zwrocic wiele grafow, ale aktualnie tylko
  # FIXME: jeden biore pod uwage w wykonywaniu algorytmu! pierwszy
  g = graphs[0]
  return builders, mergers, wsd_alg, g


if __name__ == '__main__':
  main()



#!/usr/bin/env python
# -*- coding: utf-8 -*-

#
# <one line to give the library's name and an idea of what it does.>
# Copyright (C) 2014  <copyright holder> <email>
# 
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
# 
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# 
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
# 
#

from collections import defaultdict

from graph_tool.all import *
from wosedon.algorithms.wsdalgorithminterface import WSDAlgorithmInterface
from wosedon.ranking.wsd_ranking import WSDRanking

class GTSUDOKURun2(WSDAlgorithmInterface):
  def __init__(self, str_name = 'GTSUDOKURun2'):
    super(GTSUDOKURun2, self).__init__(str_name)
    self._monosemy_set = None
    self._polysemy_dict = None
    self._subgraph = None
    self._main_nodes = None
    self._intermediate_nodes = None
    self._L = 2

  def prepare_v(self, wsd_context, graph, degree_of_polysemy):
    v = self._subgraph.new_vertex_property("double")
    for node in self._subgraph.vertices():
      old_node = self._subgraph.vp["old_node"][node]
      if old_node in self._monosemy_set:
        v[node] = self._multiply_factor
      else:
        v[node] = self._multiply_factor * (1.0 / float(degree_of_polysemy))
    return v
  
  def run(self, wsd_context, graph, options, resources = None):
    wsd_rank = WSDRanking()
    (lemma_on_node_dict, lemma_on_only_synset_node_dict) = \
      wsd_rank.get_lemma_on_node_dict(wsd_context, graph, options.ini_nodes())

    self._set_monosemy_set_and_polysemy_dict(lemma_on_only_synset_node_dict)
    self._initialize_graph()

    for degree_of_polysemy in sorted(self._polysemy_dict):
      self._clear_all()
      self._add_monosemous_nodes()
      self._add_polysemous_nodes(degree_of_polysemy)
      self._build_subgraph(graph.use_graph_tool())

      pers_v = self.prepare_v(wsd_context, graph, degree_of_polysemy)
      (ranking, ret_iter) = pagerank(self._subgraph, 
                                     pers = pers_v, 
                                     max_iter = 2 * options.max_iter(),
                                     damping = options.damping_factor(),
                                     ret_iter = True)

      self._rebuild_monosemy_set(degree_of_polysemy, 
                                 lemma_on_only_synset_node_dict,
                                 ranking)

    new_ranking = graph.ungraph_tool(self._prepare_ranking(graph.use_graph_tool(), ranking))

    for (lemma, pos_str), nodes in lemma_on_only_synset_node_dict.iteritems():
      for node in nodes:
        node = int(node)
        if node in self._main_nodes:
          wsd_rank.set_ranking_for_lemma(lemma, pos_str, new_ranking)

    return (wsd_rank, ret_iter)

  def _set_monosemy_set_and_polysemy_dict(self, lemma_on_only_synset_node_dict):
    self._monosemy_set = set()
    self._polysemy_dict = defaultdict(set)
    for nodes in lemma_on_only_synset_node_dict.itervalues():
      synset_set_size = len(nodes)
      if synset_set_size == 1:
        self._monosemy_set.add(int(next(iter(nodes))))
      else:
        for node in nodes:
          self._polysemy_dict[synset_set_size].add(int(node))

  def _initialize_graph(self):
    self._subgraph = Graph(directed = False)
    
    old_node = self._subgraph.new_vertex_property("int")
    self._subgraph.vp["old_node"] = old_node

    self._main_nodes = {}
    self._intermediate_nodes = {}

  def _add_monosemous_nodes(self):
    for node in self._monosemy_set:
      if node not in self._main_nodes:
        v = self._subgraph.add_vertex()
        self._subgraph.vp["old_node"][v] = node
        self._main_nodes[node] = v

  def _add_polysemous_nodes(self, degree_of_polysemy):
    for node in self._polysemy_dict[degree_of_polysemy]:
      if node not in self._main_nodes:
        v = self._subgraph.add_vertex()
        self._subgraph.vp["old_node"][v] = node
        self._main_nodes[node] = v

  def _get_new_node(self, old_node, stack):
    new_node = None
    if old_node in self._main_nodes:
      new_node = self._main_nodes[old_node]
    elif old_node in self._intermediate_nodes:
      new_node = self._intermediate_nodes[old_node]
    else:
      v = self._subgraph.add_vertex()
      self._subgraph.vp["old_node"][v] = old_node
      self._intermediate_nodes[old_node] = v
      new_node = v
      stack.append(old_node)
    return new_node

  def _build_subgraph(self, graph):
    stack = self._main_nodes.keys()
    while stack:
      old_node = stack.pop()
      for path in self._dfs_paths(graph.vertex(old_node)):
        for i in range(0, len(path)-1):
          parent_node = self._get_new_node(int(path[i]), stack)
          child_node = self._get_new_node(int(path[i+1]), stack)
          self._subgraph.add_edge(parent_node, child_node)
    remove_parallel_edges(self._subgraph)

  def _dfs_paths(self, v_start):
    stack = [(v_start, [v_start])]
    while stack:
      (vertex, path) = stack.pop()
      if len(path) <= self._L:
        for ngb in vertex.all_neighbours():
          if ngb not in path:
            if int(ngb) in self._main_nodes:
              yield path + [ngb]
            else:
              stack.append((ngb, path + [ngb]))

  def _rebuild_monosemy_set(self, degree_of_polysemy, lemma_on_only_synset_node_dict, ranking):
    for nodes in lemma_on_only_synset_node_dict.itervalues():
      synset_set_size = len(nodes)
      if synset_set_size == degree_of_polysemy:
        best_rank = 0.0
        best_nodes = set()
        for node in nodes:
          if int(node) not in self._main_nodes:
            continue
          new_node = self._main_nodes[int(node)]
          rank = ranking[new_node]
          if rank > best_rank:
            best_rank = rank
            best_nodes.clear()
            best_nodes.add(new_node)
          elif rank == best_rank:
            best_nodes.add(new_node)
        if len(best_nodes) == 1:
          best_node = next(iter(best_nodes))
          self._monosemy_set.add(self._subgraph.vp["old_node"][best_node])

  def _clear_all(self):
    self._subgraph.clear()
    self._main_nodes.clear()
    self._intermediate_nodes.clear()

  def _prepare_ranking(self, graph, ranking):
    new_ranking = graph.new_vertex_property("double")
    for node in self._subgraph.vertices():
      old_node = self._subgraph.vp["old_node"][node]
      new_ranking[graph.vertex(old_node)] = ranking[node]
    return new_ranking



#
# <one line to give the library's name and an idea of what it does.>
# Copyright (C) 2014  <copyright holder> <email>
# 
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
# 
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# 
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
# 
#

import os, logging
import sys
from collections import defaultdict

from wosedon.basegraph import BaseGraph
from mergerinterface import MergerInterface

class SynsetsWNDMerger(MergerInterface):
  """!
  Class for merging synsets graph and WND graph.

  WND graph was built based on the WND graph file, which is described in 
  WNDGraphBuilder class. In WND graph each node represents one domain/concept. 
  Whereas each edge represents relation occurring between them.

  The purpose of this class is to merge two graphs, namely synsets graph and 
  WND graph. Nodes from WND graph are conected to proper nodes from synsets graph, 
  based on mapping PLWN on WND file.
  """
  def __init__(self, resources, options, str_name = 'SynsetsWNDMerger'):
    """!
    @param resources - object of Resources class contain resources from config 
                       file
    @param options   - object of MergeOptions class contain merge options from 
                       config file
    @param str_name  - default value same as the class name "SynsetsWNDMerger"
    """
    super(SynsetsWNDMerger, self).__init__(resources, options, str_name)

  def get_plwn2wnd_dict(self):
    """!
    Create dictionary based on mapping PLWN on WND file. Dictionary format 
    and mapping PLWN on WND file format are presented below.

    Format of mapping PLWN on WND file:
    \code
    synset_str_plwn;domain_plwn;synset_id_plwn;interlingual_rel; \
    synset_str_pwn;pwn_synset_pos;domain_pwn;synset_id_pwn;wnd_domain
    \endcode

    Sample format of mapping PLWN on WND file:
    \code
    adres.5;por-7;16;syn_pa; \
    address.6;n;por-7;321457;geography

    adwokat.1;os-15;17;syncz_pa; \
    advocate.2;n;os-15;337414;law

    ...
    \endcode

    Format of dictionary:
    \code
    wnd_concept -> set([synset_id_1, ..., synset_id_N])
    \endcode

    Sample format of created dictionary:
    \code
    geography -> set([19247])
    law -> set([25849, 25851, 44443, 398925])
    ...
    \endcode

    @return dictionary, where key is a WND concept and value is a set of synsets 
            identifiers
    """
    if not os.path.exists(self.resources().mapping_wnd_file()):
      raise IOError(
        "%s file not found!" % \
          self.resources().mapping_wnd_file()
          )

    plwn2wnd_dict = defaultdict(set)

    with open(self.resources().mapping_wnd_file()) as wndfile:
      next(wndfile)
      for line in wndfile:
        synset_id = int(line.strip().split(';')[2])
        wnd = line.strip().split(';')[8]
        plwn2wnd_dict[wnd].add(synset_id)
    
    return plwn2wnd_dict

  def merge(self, g1, g2):
    """!
    Merge two given graphs, namely synsets graph and WND graph. The final 
    graph contain two types of nodes. Each synset node has an attribute named "synset", 
    to which is assigned synset object. Synset object consist of: synset_id and lu_set, 
    which contains lexical units objects. Whereas lexical units objects consists of: lu_id, 
    lemma, pos, variant and domain. Each WND node has an attribute named "wnd", to which is 
    assigned a WND concept. To the relation between synsets nodes is assigned relation 
    identifier from PLWN. Whereas to the relation between WND nodes is assigned 
    relation name, which describe a type of connection between linked concepts. Based 
    on mapping PLWN on WND dictionary new relations are added between synsets nodes and 
    WND nodes.

    The directness of the graph and uniqueness of edges are inherited from synsets graph.

    Warning: If the final graph is a directed graph then two edges will be added between 
    a synset node and a WND node. First edge will be add from synset node to WND node 
    and second from WND node to synset node.

    @param g1 - synsets graph
    @param g2 - WND graph
    @return object of BaseGraph class
    """
    logger = logging.getLogger(__name__)
    
    
    g = BaseGraph()
    g.init_graph(drctd = g1.is_directed())

    plwn2wnd_dict = defaultdict(set)
    plwn2wnd_dict = self.get_plwn2wnd_dict()

    g.merge_graphs(g1, g2)

    nodes_dict = {}
    for node in g.all_nodes():
      synset_id = None
      if node.synset:
        synset_id = node.synset.synset_id
      wnd = None
      if node.wnd:
        wnd = node.wnd

      if synset_id and wnd:
        logger.warning("Synset and wnd are both in the same node.")
        continue
      if synset_id:
        if nodes_dict.has_key(synset_id):
          logger.warning("ID of some synset is not unique.")
          continue
        nodes_dict[synset_id] = node
      if wnd:
        if nodes_dict.has_key(wnd):
          logger.warning("WND is not unique.")
          continue
        nodes_dict[wnd] = node

    for wnd, synset_id_set in plwn2wnd_dict.iteritems():
      if nodes_dict.has_key(wnd):
        wnd_node = nodes_dict[wnd]
        for synset_id in synset_id_set:
          if nodes_dict.has_key(synset_id):
            synset_id_node = nodes_dict[synset_id]
            g.add_edge(wnd_node, synset_id_node, [("rel", 'syn-wnd')], simply = True)
            if g.is_directed():
              g.add_edge(synset_id_node, wnd_node, [("rel", 'syn-wnd')], simply = True)
          else:
            logger.warning("The graph hasn't got synset %d, which is in the mapping file.", synset_id)
      else:
        logger.warning("The graph hasn't got wnd '%s', which is in the mapping file.", wnd)

    return g



import unittest
from wosedon.utils.resources import Resources

import corpus2
from collections import defaultdict




class TestResources(unittest.TestCase):
  
  def setUp(self):
    self.res = Resources()
  
  
  def test_tagset(self):
    # it's doubled in case it work only once
    self.res.set_tagset('nkjp')
    self.assertIsInstance(self.res.tagset(), corpus2.Tagset)
    self.assertIsInstance(self.res.tagset(), corpus2.Tagset)
    self.res.set_tagset('kipi')
    self.assertIsInstance(self.res.tagset(), corpus2.Tagset)
    self.assertIsInstance(self.res.tagset(), corpus2.Tagset)
    self.res.set_tagset('morfeusz2')
    self.assertIsInstance(self.res.tagset(), corpus2.Tagset)
    self.assertIsInstance(self.res.tagset(), corpus2.Tagset)
  
  
  def test_nottagset(self):
    self.res.set_tagset('!@#$%^')
    self.assertRaises(IndexError, self.res.tagset)
    self.res.set_tagset(354)
    self.assertRaises(TypeError, self.res.tagset)
    self.res.set_tagset(['nkjp'])
    self.assertRaises(TypeError, self.res.tagset)
    
    
  def test_glossfile(self):
    self.res.set_tagset('nkjp')
    self.res.set_gloss_file('tests/#Data/glossfile.xml')
    gf = self.res.gloss_file()
    
    self.assertIsInstance(gf, dict)
    self.assertEquals(len(gf), 5)
    self.assertEquals(sorted(gf.keys()), [10, 14, 16, 17, 18])
    for i in gf:
      self.assertIsInstance(gf[i], corpus2.ChunkPtr)
      self.assertIsInstance(gf[i], corpus2.ChunkPtr)
  
  
  def test_impedancetable(self):
    self.res.set_impedance_table('tests/#Data/impedance.csv')
    it = self.res.impedance_table()
    
    self.assertIsInstance(it, defaultdict)
    self.assertEquals(len(it), 4)
    for i in it:
      self.assertIsInstance(i, str)
      self.assertIsInstance(it[i], defaultdict)
      self.assertEquals(len(it[i]), 4)
      for j in it[i]:
        self.assertIsInstance(j, str)
        self.assertIsInstance(it[i][j], float)
    
    self.assertEquals(it[7][2], 0)
    















#
# <one line to give the library's name and an idea of what it does.>
# Copyright (C) 2014  <copyright holder> <email>
# 
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
# 
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# 
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
# 
#

import sys, logging

from wosedon.basegraph import BaseGraph
from mergerinterface import MergerInterface

class SynsetsCCLMerger(MergerInterface):
  """!
  Class for merging ccl graph and SUMO graph. Synsets from synset graf are additionaly
  connected with edges from ccl graph.
  """
  def __init__(self, resources, options, str_name = 'SynsetsCCLMerger'):
    """!
    @param resources - object of Resources class contain resources from config 
                       file
    @param options   - object of MergeOptions class contain merge options from 
                       config file
    @param str_name  - default value same as the class name "SynsetsCCLMerger"
    """
    super(SynsetsCCLMerger, self).__init__(resources, options, str_name)


  def merge(self, g1, g2):
    """!
    @param g1 - synsets graph
    @param g2 - CCL graph
    @return object of BaseGraph class
    """
    g = BaseGraph()
    g.copy_graph_from(g1)
    
    for edge in g2.all_edges():
      source_node = g.get_node_for_synset_id(edge.source().synset_id)
      target_node = g.get_node_for_synset_id(edge.target().synset_id)
      
      if not source_node:
        logging.getLogger(__name__).warning("Ccl graph contains unknown synset: %d", edge.source().synset_id)
        
      if not target_node:
        logging.getLogger(__name__).warning("Ccl graph contains unknown synset: %d", edge.target().synset_id)
      
      if source_node and target_node:
        ne = g.add_edge(source_node, target_node, simply=True)
        ne.rel = edge.rel

    return g






























#
# <one line to give the library's name and an idea of what it does.>
# Copyright (C) 2014  <copyright holder> <email>
# 
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
# 
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# 
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
# 
#

class LeskFilterOptions:
  def __init__(self):
    self._input_file = None
    self._allow_only = False
  
  def set_input_file(self, path):
    self._input_file = path
  
  def set_allow_only(self, val):
    self._allow_only = val
  
  def input_file(self):
    return self._input_file
  
  def allow_only(self):
    return self._allow_only
  



# -*- coding: utf-8 -*-

import unittest, re
from wosedon.utils.resources import Resources
from wosedon.utils.buildoptions import BuildOptions
from wosedon.utils.mergeoptions import MergeOptions
from wosedon.gbuilders import SynsetGraphBuilder
from wosedon.gbuilders import LexicalUnitGraphBuilder
from wosedon.mergers import SynsetsLUMerger





def leniter(i):
  """ Lenght of any iterable """
  return sum(1 for e in i)



class TestSynsetsLUMerger_DirectnessAndDuplicates(unittest.TestCase):
  
  def setUp(self):
    res = Resources()
    res.set_plwn_graph_file('tests/#Data/plwnik')
    
    bopts = BuildOptions()
    bopts.set_directed_graph(False)
    bopts.set_unique_edges(True)
    
    mopts = MergeOptions()
    
    gs = SynsetGraphBuilder(res, bopts).build_graph()
    gl = LexicalUnitGraphBuilder(res, bopts).build_graph()
    self.graph = SynsetsLUMerger(res, mopts).merge(gs, gl)
  
  
  def test_undirected_unique(self):
    for s in self.graph.all_nodes():
      for t in self.graph.all_nodes():
        if s != t:
          if leniter(self.graph.get_edges_between(s, t)) > 1:
            self.fail((s, t))
          self.assertLessEqual(leniter(self.graph.get_edges_between(s, t)), 1)
        



class TestSynsetsLUMerger(unittest.TestCase):
  
  def setUp(self):
    res = Resources()
    res.set_plwn_graph_file('tests/#Data/plwnik')
    
    bopts = BuildOptions()
    bopts.set_directed_graph(True)
    bopts.set_unique_edges(False)
    
    mopts = MergeOptions()
    
    gs = SynsetGraphBuilder(res, bopts).build_graph()
    gl = LexicalUnitGraphBuilder(res, bopts).build_graph()
    self.graph = SynsetsLUMerger(res, mopts).merge(gs, gl)
  
  
  def test_edge_names(self):
    for e in self.graph.all_edges():
      self.assertTrue(e.rel == 'syn-lu' or re.match('[sl]\d+', e.rel))
  
  
  def test_node_types(self):
    for n in self.graph.all_nodes():
      if n.synset:
        self.assertIsInstance(n.synset.synset_id, int)
        self.assertIsInstance(n.synset.lu_set, set)
      elif n.lu:
        self.assertIsInstance(n.lu.lu_id, int)
        self.assertIsInstance(n.lu.lemma, str)
      else:
        self.fail('The node no %d is neither synset nor lu!' % int(n))
  
  
  def test_numbers(self):
    self.assertEquals(leniter(self.graph.all_nodes()), 22)
    self.assertEquals(leniter(self.graph.all_edges()), 54)






























from wosedon.lesk_filters.leskfilterinterface import LeskFilterInterface



class Yes(LeskFilterInterface):
  """! Allows everything. """
  
  def __init__(self, tagset, options, str_name='Yes'):
    super(Yes, self).__init__(tagset, options, str_name)
  
  def __contains__(self, lexeme):
    return True



#!/usr/bin/env python
# -*- coding: utf-8 -*-

from setuptools import setup

setup(name='CclExtractor',
	version='0.1.0',
	description='Extract synset graph from .ccl files.',
	author= "Michał Moczulski",
	author_email="michalmoczulski1a@gmail.com",
        packages=[
            'CclExtractor'
	],
	license='', 
	entry_points={
		'console_scripts': [
			'CclExtractor = CclExtractor.CclExtractor:main'
		]
	}
)



from wosedon.reranks.rerankinterface import RerankInterface
import operator

class AContentReranker(RerankInterface):
  def __init__(self, str_name = 'AContentReranker'):
    super(AContentReranker, self).__init__(str_name)

  def rerank(self, ranking, graph, options):
    new_ranking = []
    for (token, token_rank) in ranking:
      if not token_rank:
        new_ranking.append((token, token_rank))
        continue

      new_token_rank = []
      node = token_rank[0][0]
      for ngb in node.all_neighbours():
        lu_set = ngb.synset.lu_set
        lexicon = self._get_most_frequent_lexicon(graph, lu_set)
        if lexicon == "AContent_1.0":
          new_token_rank.append((ngb, 0.0))
      new_ranking.append((token, new_token_rank))

    return new_ranking

  def _get_most_frequent_lexicon(self, graph, lu_set):
    lexicon_dict = {}
    for lu in lu_set:
      if lu.lexicon not in lexicon_dict:
        lexicon_dict[lu.lexicon] = 0
      lexicon_dict[lu.lexicon] += 1
    return max(lexicon_dict.iteritems(), key=operator.itemgetter(1))[0]



#
# <one line to give the library's name and an idea of what it does.>
# Copyright (C) 2014  <copyright holder> <email>
# 
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
# 
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# 
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
# 
#

from graph_tool.centrality import pagerank
from wosedon.algorithms.wsdalgorithminterface import WSDAlgorithmInterface
from wosedon.ranking.wsd_ranking import WSDRanking

class GTStaticPR(WSDAlgorithmInterface):
  """!
  Static Page Rank is a abbreviation from GraphToolStatic PageRank. 
  In this imeplementation, Graph tool Page Rank is used.
  This is Static Page Rank so no personalization is done.
  """
  
  def __init__(self, str_name = 'GTStaticPR'):
    """!
    @param str_name - default value same as the class name "GTStaticPR"
    """
    super(GTStaticPR, self).__init__(str_name)

  def prepare_v(self, wsd_context, graph):
    """!
    Personalization vector "v" is made for all nodes into graph.
    All nodes has same value equal to 1/N where N is the number of
    nodes into Graph.

    @param wsd_context - object of WSDContext class
    @param graph - object of BaseGraph class
    @return PropertyMap (see: http://graph-tool.skewed.de/static/doc/graph_tool.html?highlight=new_vertex_property#graph_tool.Graph.new_vertex_property) of personalized vector "v"
    """
    w = (1.0 / float(graph.use_graph_tool().num_vertices())) * self._multiply_factor

    v = graph.use_graph_tool().new_vertex_property("double")
    for node in graph.use_graph_tool().vertices():
      v[node] = w
    # v.set_value(w)
    
    return v
  
  def run(self, wsd_context, graph, options, resources):
    """!
    Disambiguate analyzed context.

    @param wsd_context - object of WSDContext class
    @param graph - object of BaseGraph class
    @param options - object of AlgorithmOptions class
    @param resources - object of Resources class
    @return tuple of object of WSDRanking class and number of algorithm iterations
    """
    wsd_rank = WSDRanking()
    (lemma_on_node_dict, lemma_on_only_synset_node_dict) = \
      wsd_rank.get_lemma_on_node_dict(wsd_context, graph, options.ini_nodes())

    pers_v = self.prepare_v(wsd_context, graph)
    (ranking, ret_iter) = pagerank(graph.use_graph_tool(), 
                                   pers = pers_v, 
                                   max_iter = 2 * options.max_iter(),
                                   damping = options.damping_factor(),
                                   ret_iter = True,
                                   weight = graph.use_graph_tool().ep["weight"])
    
    ranking = graph.ungraph_tool(ranking)
    for (lemma, pos_str) in lemma_on_only_synset_node_dict.iterkeys():
      wsd_rank.set_ranking_for_lemma(lemma, pos_str, ranking)

    return (wsd_rank, ret_iter)



import subprocess
import sys
import importlib
DOCKER_IMAGE_NAME = "styczynski/eniam:0.1.1"

from base64 import b64encode
import time
import sys
from subprocess import PIPE, Popen
from threading  import Thread
import tempfile

try:
    from queue import Queue, Empty
except ImportError:
    from Queue import Queue, Empty  # python 2.x

ON_POSIX = 'posix' in sys.builtin_module_names

def enqueue_output(out, queue):
    for line in iter(out.readline, b''):
        queue.put(line)
    out.close()

def create_cmd(args_list):
    return " ".join(args_list)

def run_subsyntax(args=[], input=None, output=None, lexicon=None):

    lexicon_input = "empty" if lexicon is None else b64encode(lexicon.encode()).decode()
    cmd = create_cmd(["docker", "run", "-i", DOCKER_IMAGE_NAME, "/root/subsyntax.sh", lexicon_input] + args),
    return subprocess.Popen(
        cmd,
        stdin=input,
        stdout=output,
        stderr=output,
        shell=True,
        executable='/bin/bash',
        bufsize=1,
        close_fds=ON_POSIX,
    )


def run_lex(args=[], input=None, output=None):
    return subprocess.Popen(
        create_cmd(["docker", "run", "-i", DOCKER_IMAGE_NAME, "/root/print_lexicon"] + args),
        stdin=input,
        stdout=output,
        stderr=output,
        shell=True,
        executable='/bin/bash',
        bufsize=1,
        close_fds=ON_POSIX,
    )


def run_domparser(args=[], input=None, output=None, lexicon=None, valence=None):
    lexicon_input = "empty" if lexicon is None else b64encode(lexicon.encode()).decode()
    valence_input = "empty" if valence is None else b64encode(valence.encode()).decode()
    return subprocess.Popen(
        create_cmd(["docker", "run", "-i", DOCKER_IMAGE_NAME, "/root/domparser.sh", lexicon_input, valence_input] + args),
        stdin=input,
        stdout=output,
        stderr=output,
        shell=True,
        executable='/bin/bash',
        bufsize=1,
        close_fds=ON_POSIX,
    )


def csv_to_html_table(fname,headers=None,delimiter=","):
    with open(fname) as f:
        content = f.readlines()
    #reading file content into list
    rows = [x.strip() for x in content]
    table = "<table>"
    #creating HTML header row if header is provided
    if headers is not None:
        table+= "".join(["<th>"+cell+"</th>" for cell in headers.split(delimiter)])
    else:
        table+= "".join(["<th>"+cell+"</th>" for cell in rows[0].split(delimiter)])
        rows=rows[1:]
    #Converting csv to html row by row
    for row in rows:
        table+= "<tr>" + "".join(["<td>"+cell+"</td>" for cell in row.split(delimiter)]) + "</tr>" + "\n"
    table+="</table><br>"
    return table

class DomparserResult():
    def __init__(self, html_content, output_pdfs, output_pngs, file_map):
        self.html_conent = html_content
        self.output_pdfs = output_pdfs
        self.output_pngs = output_pngs
        self.file_map = file_map
        for file in self.output_pdfs:
            file.seek(0)
        for file in self.output_pngs:
            file.seek(0)

    def _replace_all_links(self, html_code):
        for original_file_name in self.file_map:
            entry = self.file_map[original_file_name]
            path = original_file_name.split('/')
            path = path[len(path)-1].replace('\n', '').replace('\r', '')
            if entry[0] == 'png':
                html_code = html_code.replace(f'<A HREF="{path}">', self.output_chart_html(entry[1]) + "<a style='display:none;'>")
            else:
                html_code = html_code.replace(f'<A HREF="{path}">', self.html_chart(entry[1]) + "<a style='display:none;'>")
        return html_code

    def show(self):
        display = importlib.import_module("IPython.core.display")
        display.display(display.HTML(self._replace_all_links("\n".join(self.html_conent))))

    def html(self):
        return self.html_conent

    def output_chart_html(self, index=0):
        self.output_pngs[index].seek(0)

        image_data = None
        with open(self.output_pngs[index].name, "rb") as img_file:
            image_data = b64encode(img_file.read()).decode()

        return f'<div><img src="data:image/png;base64,{image_data}"/></div>'

    def output_chart(self, index=0):
        display = importlib.import_module("IPython.core.display")
        display.display(display.HTML(self.output_chart_html(index)))

    def chart(self, index=0):
        self.output_pdfs[index].seek(0)
        WImage = importlib.import_module("wand.image").Image
        img = WImage(filename=self.output_pdfs[index].name)
        return img

    def tables_chart(self, index=0):
        self.output_pdfs[index].seek(0)
        tables = importlib.import_module("camelot").read_pdf(self.output_pdfs[index].name)
        return tables

    def csv_chart_file(self, index=0):
        output_csv = tempfile.NamedTemporaryFile(suffix='.csv')
        tables = self.tables_chart(index)
        tables[0].to_csv(output_csv.name)
        output_csv.seek(0)
        return output_csv

    def csv_chart(self, index=0):
        f = self.csv_chart_file(index)
        output = f.read().decode()
        f.close()
        return output

    def html_chart(self, index=0):
        tables = self.tables_chart(index)
        content = tables[0].df.to_html()
        return content


class SubsyntaxResult():
    def __init__(self, html_content):
        self.html_conent = html_content

    def show(self):
        display = importlib.import_module("IPython.core.display")
        display.display(display.HTML("\n".join(self.html_conent)))

    def html(self):
        return self.html_conent


class Eniam():
    def __init__(self, lexicon_keywords=None, lexicon=None, valence_names=None, valence=None):

        lexicon_base_keywords = """
          infp np prepnp adjp ip cp ncp advp padvp
          adja prepadjp compar measure num aglt aux-fut
          aux-past aux-imp qub interj hyphen int
          rparen rparen2 rquot rquot2 rquot3 inclusion
          day-interval day-lex day-month-interval date-interval
          month-lex month-interval year-interval roman roman-interval
          hour-minute-interval hour-interval obj-id match-result
          url email day-month day year date hour hour-minute
          się nie by s <root> <conll_root> or or2 <colon> <speaker> <speaker-end> <squery> <sentence> <paragraph>
          <subst> <depr> <ppron12> <ppron3> <siebie> <prep> <num> <numcomp> <intnum>
          <realnum> <intnum-interval> <realnum-interval> <symbol> <ordnum>
          <date> <date-interval> <hour-minute> <hour> <hour-minute-interval>
          <hour-interval> <year> <year-interval> <day> <day-interval> <day-month>
          <day-month-interval> <month-interval> <roman> <roman-interval> <roman-ordnum>
          <match-result> <url> <email> <phone-number> <postal-code> <obj-id> <list-item> <fixed> <adj> <apron> <adjc> <adjp> <adja>
          <adv> <ger> <pact> <ppas> <fin> <bedzie> <praet> <winien> <impt>
          <imps> <pred> <aglt> <inf> <pcon> <pant> <qub> <comp> <compar> <conj> <interj>
          <sinterj> <burk> <interp> <part> <unk> <building-number> do w na location time link
        """

        lexicon_base = """
            pos=subst:  <subst>;
            pos=year:  <year>;
            pos=year-interval:  <year-interval>;
            pos=prep:  <prep>;
            pos=depr:  <depr>;
            pos=ppron12:  <ppron12>;
            pos=ppron3:  <ppron3>;
            pos=siebie:  <siebie>;
            pos=num:  <num>;
            pos=numcomp:  <numcomp>;
            pos=intnum:  <intnum>;
            pos=realnum:  <realnum>;
            pos=intnum-interval:  <intnum-interval>;
            pos=realnum-interval:  <realnum-interval>;
            pos=symbol:  <symbol>;
            pos=ordnum:  <ordnum>;
            pos=date:  <date>;
            pos=date-interval:  <date-interval>;
            pos=hour-minute:  <hour-minute>;
            pos=hour:  <hour>;
            pos=hour-minute-interval:  <hour-minute-interval>;
            pos=hour-interval:  <hour-interval>;
            pos=day:  <day>;
            pos=day-interval:  <day-interval>;
            pos=day-month:  <day-month>;
            pos=day-month-interval:  <day-month-interval>;
            pos=month-interval:  <month-interval>;
            pos=roman:  <roman>;
            pos=roman-interval:  <roman-interval>;
            pos=roman-ordnum:  <roman-ordnum>;
            pos=match-result:  <match-result>;
            pos=building-number:  <building-number>;
            pos=url:  <url>;
            pos=email:  <email>;
            pos=phone-number:  <phone-number>;
            pos=postal-code:  <postal-code>;
            pos=list-item:  <list-item>;
            pos=obj-id:  <obj-id>;
            pos=fixed:  <fixed>;
            pos=apron:  <apron>;
            pos=adj:  <adj>;
            pos=adjc:  <adjc>;
            pos=adjp:  <adjp>;
            pos=adja:  <adja>;
            pos=adv:  <adv>;
            pos=ger:  <ger>;
            pos=pact:  <pact>;
            pos=ppas:  <ppas>;
            pos=fin:  <fin>;
            pos=bedzie:  <bedzie>;
            pos=praet:  <praet>;
            pos=winien:  <winien>;
            pos=impt:  <impt>;
            pos=imps:  <imps>;
            pos=pred:  <pred>;
            pos=aglt:  <aglt>;
            pos=inf:  <inf>;
            pos=pcon:  <pcon>;
            pos=pant:  <pant>;
            pos=qub:  <qub>;
            pos=comp:  <comp>;
            pos=conj:  <conj>;
            pos=interj:  <interj>;
            pos=sinterj:  <sinterj>;
            pos=burk:  <burk>;
            pos=interp:  <interp>;
            pos=part:  <part>;
            pos=compar:  <compar>;
            pos=unk:  <unk>;
            pos=subst:  <subst>;
            pos=year:  <year>;
            pos=year-interval:  <year-interval>;
            pos=prep:  <prep>;
            pos=depr:  <depr>;
            pos=ppron12:  <ppron12>;
            pos=ppron3:  <ppron3>;
            pos=siebie:  <siebie>;
            pos=num:  <num>;
            pos=numcomp:  <numcomp>;
            pos=intnum:  <intnum>;
            pos=realnum:  <realnum>;
            pos=intnum-interval:  <intnum-interval>;
            pos=realnum-interval:  <realnum-interval>;
            pos=symbol:  <symbol>;
            pos=ordnum:  <ordnum>;
            pos=date:  <date>;
            pos=date-interval:  <date-interval>;
            pos=hour-minute:  <hour-minute>;
            pos=hour:  <hour>;
            pos=hour-minute-interval:  <hour-minute-interval>;
            pos=hour-interval:  <hour-interval>;
            pos=day:  <day>;
            pos=day-interval:  <day-interval>;
            pos=day-month:  <day-month>;
            pos=day-month-interval:  <day-month-interval>;
            pos=month-interval:  <month-interval>;
            pos=roman:  <roman>;
            pos=roman-interval:  <roman-interval>;
            pos=roman-ordnum:  <roman-ordnum>;
            pos=match-result:  <match-result>;
            pos=building-number:  <building-number>;
            pos=url:  <url>;
            pos=email:  <email>;
            pos=phone-number:  <phone-number>;
            pos=postal-code:  <postal-code>;
            pos=list-item:  <list-item>;
            pos=obj-id:  <obj-id>;
            pos=fixed:  <fixed>;
            pos=apron:  <apron>;
            pos=adj:  <adj>;
            pos=adjc:  <adjc>;
            pos=adjp:  <adjp>;
            pos=adja:  <adja>;
            pos=adv:  <adv>;
            pos=ger:  <ger>;
            pos=pact:  <pact>;
            pos=ppas:  <ppas>;
            pos=fin:  <fin>;
            pos=bedzie:  <bedzie>;
            pos=praet:  <praet>;
            pos=winien:  <winien>;
            pos=impt:  <impt>;
            pos=imps:  <imps>;
            pos=pred:  <pred>;
            pos=aglt:  <aglt>;
            pos=inf:  <inf>;
            pos=pcon:  <pcon>;
            pos=pant:  <pant>;
            pos=qub:  <qub>;
            pos=comp:  <comp>;
            pos=conj:  <conj>;
            pos=interj:  <interj>;
            pos=sinterj:  <sinterj>;
            pos=burk:  <burk>;
            pos=interp:  <interp>;
            pos=part:  <part>;
            pos=compar:  <compar>;
            pos=unk:  <unk>;
        """

        if lexicon is None:
            lexicon = dict()

        if lexicon_keywords is None:
            lexicon_keywords = []

        if valence_names is None:
            valence_names = []

        if valence is None:
            valence = dict()

        lexicon_custom = []
        for key in lexicon:
            if key == "__root__":
                lexicon_custom.append(f"lemma=</sentence>,pos=interp:     BRACKET {lexicon[key]};")
            else:
                lexicon_custom.append(f"{key}: {lexicon[key]};")

        lexicon_custom_keywords_str = " ".join(lexicon_keywords)
        lexicon_custom_str = "\n".join(lexicon_custom)
        self.lexicon_content = f"""
            @PHRASE_NAMES
                {lexicon_base_keywords}
                {lexicon_custom_keywords_str}
            
            @WEIGHTS
            
            @LEXICON
            
                lemma=<sentence>,pos=interp:      BRACKET <root>/s;
            
                {lexicon_custom_str}
                {lexicon_base}
        """

        valence_rules = []
        for key in valence:
            valence_rules.append(f"{key}:\t{valence[key]}: ;")

        valence_names_str = " ".join(valence_names)
        valence_rules_str = "\n".join(valence_rules)
        self.valence_content = f"""
        @PARAM_NAMES

          jak czy za do po o w na z u dla przeciwko celu żeby że bo jaki który to ile  aby
        
        @SELPREF_NAMES
        
        {valence_names_str}
        
        @ROLE_NAMES
        
        @LEXICON
        
        {valence_rules_str}
        """

    def dom(self, input_text, silent=True):
        p = run_domparser(["--no-sem", "--no-def-cat"], PIPE, PIPE, self.lexicon_content, self.valence_content)
        q_out = Queue()
        q_err = Queue()

        p.stdin.write((input_text + "\r\n\r\n\r\n\r\n").encode())
        p.stdin.close()

        t = Thread(target=enqueue_output, args=(p.stdout, q_out))
        t.daemon = True  # thread dies with the program
        t.start()

        t_err = Thread(target=enqueue_output, args=(p.stderr, q_err))
        t_err.daemon = True  # thread dies with the program
        t_err.start()

        start_time = None
        output = []
        collect_pdf = False
        collect_png = False
        collect_output = False
        collect_err = False
        output_err = []

        pdf_files = []
        png_files = []

        file_map = dict()

        pipe_start_time = time.time()
        while True:
            try:
                line = q_out.get_nowait()
            except Empty:
                if start_time is not None:
                    if (time.time() - start_time > 3):
                        break
                if (time.time() - pipe_start_time > 10) and (start_time is None):
                    raise Exception("Parser timeout")
                time.sleep(1)
                #print("scan")
            else:
                if start_time is None:
                    start_time = time.time()
                line_content = ""
                if (not collect_pdf) and (not collect_png):
                    line_content = line.decode()
                    if line_content.find("LCG lexicon error") > -1:
                        collect_err = True
                    #print("=> "+line_content)
                if collect_err:
                    output_err.append(line_content)
                if line_content.startswith("<!DOCTYPE"):
                    collect_output = True
                if line_content.startswith("@end_png@"):
                    collect_png = False
                else:
                    if collect_png:
                        png_files[len(png_files)-1].write(line)
                    if line_content.startswith("@begin_png@:"):
                        new_file = tempfile.NamedTemporaryFile(suffix='.png')
                        file_map[line_content.replace("@begin_png@: ", "")] = ["png", len(png_files)]
                        collect_output = False
                        collect_png = True
                        png_files.append(new_file)
                        #print("collect png mode")
                if line_content.startswith("@end_pdf@"):
                    collect_pdf = False
                else:
                    if collect_pdf:
                        pdf_files[len(pdf_files)-1].write(line)
                    if line_content.startswith("@begin_pdf@:"):
                        new_file = tempfile.NamedTemporaryFile(suffix='.pdf')
                        file_map[line_content.replace("@begin_pdf@: ", "")] = ["pdf", len(pdf_files)]
                        collect_output = False
                        collect_pdf = True
                        pdf_files.append(new_file)
                        #print("collect pdf mode")
                if collect_output:
                    output.append(line_content)
            err_line = ""
            try:
                err_line = q_err.get_nowait().decode()
            except Empty:
                err_line = ""
            if len(err_line) > 0:
                if not silent:
                    print(err_line.replace("\n", "").replace("\r", ""))
                #collect_err = True
                #output_err.append(err_line)
                #break
        p.kill()
        time.sleep(1)
        t.join()
        t_err.join()

        if collect_err:
            raise Exception("\n".join(output_err))

        return DomparserResult(output, pdf_files, png_files, file_map)

    def syntax(self, input_text):
        p = run_subsyntax(["-h"], PIPE, PIPE, self.lexicon_content)
        q_out = Queue()

        p.stdin.write((input_text + "\r\n\r\n").encode())
        p.stdin.close()

        t = Thread(target=enqueue_output, args=(p.stdout, q_out))
        t.daemon = True  # thread dies with the program
        t.start()

        start_time = None
        output = []
        while True:
            try:
                line = q_out.get_nowait()
            except Empty:
                if start_time is not None:
                    if (time.time() - start_time > 3):
                        break
                time.sleep(1)
            else:
                if start_time is None:
                    start_time = time.time()
                output.append(line.decode())
        p.kill()
        time.sleep(1)
        t.join()
        return SubsyntaxResult(output)


from npsemrel.carrot.db import db
from collections import defaultdict
import codecs, os, sys, argparse

import platform, ctypes

if 'Linux' in platform.system():
    dlflags = sys.getdlopenflags()
    sys.setdlopenflags(dlflags | ctypes.RTLD_GLOBAL)

import maca

if 'Linux' in platform.system():
    # get back to default dlopen policy
    sys.setdlopenflags(dlflags)

import corpus2
from wcrft2 import Tagger as Wcrft2Tagger

def make_parser():
  desc = 'Generuje plik XML dla gloss z PLWN.'
  parser = argparse.ArgumentParser(description = desc)
  parser.add_argument('-d', '--db-config', dest = 'db_config', required = True)
  parser.add_argument('-i', '--in-gloss-file', dest = 'in_gloss_file', required = True)
  parser.add_argument('-o', '--out-ccl', dest = 'out_ccl', required = True)
  parser.add_argument('-t', '--tagset', dest = 'tagset', default = 'nkjp')
  return parser


def get_lu_on_gloss_dict(gloss_file):
  '''
  Format gloss_file:
  lu_1 [TABULACJA] gloss
  lu_2 [TABULACJA] gloss
  ...
  lu_N [TABULACJA] gloss
  '''
  lu_on_gloss_dict = defaultdict(set)

  with open(gloss_file, 'rt') as ifile:
    for line in ifile:
      lu_id = int(line.strip().split('\t')[0])
      gloss = line.strip().split('\t')[1]

      lu_on_gloss_dict[lu_id].add(gloss)

  return lu_on_gloss_dict

def get_bzn_synset_dict(dbconnection):
  bzn_synset_dict = defaultdict(set)

  query = "SELECT parent_id, child_id " \
          "FROM synsetrelation " \
          "WHERE rel_id=60;"
  cursor = dbconnection.cursor()
  cursor.execute(query)

  for row in cursor.fetchall():
    parent_id = int(row[0])
    child_id = int(row[1])
    bzn_synset_dict[parent_id].add(child_id)

  return bzn_synset_dict

def get_synset_on_lu_dict(dbconnection, bzn_synset_dict):
  synset_on_lu_dict = defaultdict(set)

  query = "SELECT SYN_ID, LEX_ID " \
          "FROM unitandsynset;"
  cursor = dbconnection.cursor()
  cursor.execute(query)

  for row in cursor.fetchall():
    syn_id = int(row[0])
    lu_id = int(row[1])
    synset_on_lu_dict[syn_id].add(lu_id)

  # uzupelnienie slownika synset_on_lu_dict
  # jesli pomiedzy synsetami zachodzi relacja bliskoznacznosci
  # to jednostki leksykalne sa przepisywane z jednego synsetu do drugiego
  for parent_id, child_set in bzn_synset_dict.iteritems():
    if not synset_on_lu_dict.has_key(parent_id):
      continue
    for child_id in child_set:
      if not synset_on_lu_dict.has_key(child_id):
        continue
      for lu_id in synset_on_lu_dict[child_id]:
        synset_on_lu_dict[parent_id].add(lu_id)

  return synset_on_lu_dict

def create_document(synset_on_lu_dict, lu_on_gloss_dict):
  tagger = Wcrft2Tagger("nkjp_e2.ini", "model_nkjp10_wcrft_e2")
  tagger.load_model()

  tagged_document = corpus2.Document()
  sentence_number = 0
  
  for synset_id, lu_set in synset_on_lu_dict.iteritems():
    tagged_chunk_ptr = corpus2.Chunk().clone_shared()
    tagged_chunk_ptr.set_attribute('id', str(synset_id))

    for lu_id in lu_set:
      merged_gloss = None
      if not lu_on_gloss_dict.has_key(lu_id):
        continue
      merged_gloss = ' '.join(list(lu_on_gloss_dict[lu_id]))
  
      maca_text_reader = maca.PlainTextReader.create_string_reader(merged_gloss, tagger.get_maca_config())
      for sentence in sentences(maca_text_reader):
        sentence_ptr = corpus2.AnnotatedSentence.wrap_sentence(sentence).clone_shared()
        sentence_ptr.set_id('s' + str(sentence_number) + ':' + str(lu_id))
        tagger.tag_sentence(sentence_ptr, False)
        tagged_chunk_ptr.append(sentence_ptr)
        sentence_number += 1

    if tagged_chunk_ptr.empty():
      continue

    tagged_document.add_paragraph(tagged_chunk_ptr)

  return tagged_document

def sentences(rdr):
  while True:
    sent = rdr.get_next_sentence()
    if not sent:
      break
    yield sent

def write_document(document, out_ccl, tagset):
  ccl_writer = corpus2.TokenWriter.create_path_writer(
    'ccl', 
    out_ccl, 
    tagset)
  for chunk in document.paragraphs():
    ccl_writer.write_chunk(chunk)
  ccl_writer.finish()
  del ccl_writer


def main(argv = None):
  parser = make_parser()
  args = parser.parse_args(argv)
  tagset = corpus2.get_named_tagset(args.tagset)

  print >> sys.stderr, 'Connecting to DB...',
  dbcon = db.DB()
  dbconnection = dbcon.connect(args.db_config)
  if not dbconnection:
    print >> sys.stderr, 'Cannot connect to DB!'
    exit(1)
  print >> sys.stderr, ' Done!'

  lu_on_gloss_dict = get_lu_on_gloss_dict(args.in_gloss_file)
  bzn_synset_dict = get_bzn_synset_dict(dbconnection)
  synset_on_lu_dict = get_synset_on_lu_dict(dbconnection, bzn_synset_dict)
  
  tagged_document = create_document(synset_on_lu_dict, lu_on_gloss_dict)
  write_document(tagged_document, args.out_ccl, tagset)


if __name__ == '__main__':
  main()


#
# <one line to give the library's name and an idea of what it does.>
# Copyright (C) 2014  <copyright holder> <email>
# 
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
# 
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# 
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
# 
#

import abc
import corpus2

class Context(object):
  def __init__(self, tagset, ccl_doc, ccl_rel, str_name):
    self._ccl_doc = ccl_doc
    self._ccl_rel = ccl_rel
    self._str_name = str_name
    self._tagset = tagset

  def __str__(self):
    return self._str_name
  
  def ccl_document(self):
    return self._ccl_doc
  
  def ccl_relations(self):
    return self._ccl_rel

  @abc.abstractmethod
  def reset(self):
    raise NotImplementedError("Method \"reset\" in Context is virtual!")

  @abc.abstractmethod
  def next(self):
    '''
    Implementation of @next metho should to return WSDContext
    containing tokens with ordering same as in the text
    '''
    raise NotImplementedError("Method \"next\" in Context is virtual!")

class WSDContext(object):
  '''
  Kontekst dla metody ujednoznaczniajacej tekst.
  W sklad kontekstu wchodzi lista tokentow oraz mozliwe 
  operacje na kontekscie.
  '''
  def __init__(self, tagset, tokens):
    self._tokens = tokens
    self._tagset = tagset
    self._full_pos_mask = corpus2.get_attribute_mask(tagset, '') 
  
  def tagset(self):
    return self._tagset
  
  def tokens(self):
    return self._tokens

  def get_token_lemma_str(self, token):
    return str(token.get_preferred_lexeme(self._tagset).lemma())

  def get_token_coarse_pos(self, token):
    tag = token.get_preferred_lexeme(self._tagset).tag()
    pos_mask = tag.get_masked(self._full_pos_mask)
    return self.convert_to_coarse_pos(
      self._tagset.tag_to_symbol_string(pos_mask))

  def convert_to_coarse_pos(self, pl_pos):
    noun_pl_pos = ['subst', 'depr', 'ger', 'brev']
    adj_pl_pos = ['adj', 'adja', 'adjp', 'adjc']
    verb_pl_pos = ['fin', 'bedzie', 'praet', 'impt', \
                   'inf', 'pcon', 'pant', 'imps', \
                   'winien', 'pred', 'pact', 'ppas', 'pred']
    pos_str = None
    if pl_pos in noun_pl_pos:
      pos_str = 'n'
    elif pl_pos in adj_pl_pos:
      pos_str = 'a'
    elif pl_pos in verb_pl_pos:
      pos_str = 'v'
    return pos_str

  def convert_num_pos_to_coarse_str(self, pos_int):
    pos_str = None
    if pos_int == 1:
      #verb
      pos_str = 'v'
    elif pos_int == 2:
      #subst
      pos_str = 'n'
    elif pos_int == 3:
      #adv
      pos_str = 'r'
    elif pos_int == 4:
      #adj
      pos_str = 'a'
    elif pos_int == 5:
      #PWN verb
      pos_str = 'v-PWN'
    elif pos_int == 6:
      #PWN subst
      pos_str = 'n-PWN'
    elif pos_int == 7:
      #PWN adv
      pos_str = 'r-PWN'
    elif pos_int == 8:
      #PWN adj
      pos_str = 'a-PWN'
    return pos_str



# -*- coding: utf-8 -*-

import unittest
from wosedon.lesk_filters import Yes
import corpus2




def leniter(i):
  """ Lenght of any iterable """
  return sum(1 for e in i)


class TestLeskFilterYes(unittest.TestCase):

  def setUp(self):
    self.filter = Yes(corpus2.get_named_tagset('nkjp'), {})

  def test_it(self):
    self.assertIn('abcd', self.filter)
    self.assertIn('fgdesrg', self.filter)
    self.assertIn('ab24232cd', self.filter)
    self.assertIn('8ki67j', self.filter)
    self.assertIn('rtert', self.filter)
    self.assertIn('232 4 345', self.filter)






# -*- coding: utf-8 -*-

import argparse
try:
  import argcomplete
except ImportError:
  argcomplete = None

import codecs, os, sys
import operator
from collections import defaultdict

import threading
import corpus2

from wosedon.wosedon_plugin import WoSeDonPlugin
import domains as dom

def make_parser():
  desc = 'Klasyfikator podejmujący decyzje na podstawie decyzji wielu modeli.'
  aparser = argparse.ArgumentParser(description=desc)

  aparser.add_argument('-f', '--cclfile', dest='ccl', required=True,
                        type=str, help='Ccl file.')

  aparser.add_argument('-c', '--config-files', nargs='*', action='append',
                        dest='config_files', required=True, 
                        help='WoSeDon config files.')

  aparser.add_argument('-md', '--models-dir', nargs='*', action='append',
                       dest='models_dir', required=False, 
                       help=(
                        'Output directory where the models will be stored. '
                        'If is not given then the models will not be stored. '
                        'If output directory is given, and the models '
                        'exists into directory, then tey will be loaded.'))

  aparser.add_argument('-V', '--verbose', dest='verbose', action='store_true',
                       help='If -V is set then verbose mode is enabled.')

  if argcomplete:
    argcomplete.autocomplete(aparser)
  return aparser

def get_files_paths_list(old_files_paths):
  new_files_paths_list = []
  for files_paths_list in old_files_paths:
    for file_path in files_paths_list:
      new_files_paths_list.append(file_path)
  return new_files_paths_list

def get_fpaths_list(fpaths_file):
  fpaths_list = []
  with codecs.open(fpaths_file, 'r') as ifile:
    for line in ifile:
      corpfile_full_path = os.path.join(os.path.dirname(fpaths_file), line).strip()
      fpaths_list.append(corpfile_full_path)
  return fpaths_list

def vote(best_nodes_list, model_object_NOUN, model_object_VERB):
  # identyfikator synsetu dla modelu zwracajacego najwyzsza
  # precyzje ujednoznaczniania dla rzeczownikow
  synset_id_NOUN = None
  # identyfikator synsetu dla modelu zwracajacego najwyzsza
  # precyzje ujednoznaczniania dla czasownikow
  synset_id_VERB = None

  # czesc mowy
  POS = None

  # slownik zbierajacy liczbe wystapien identyfikatorow synsetow,
  # ktore zostaly przypisane jako najlepsze w procesie WSD
  # dla kazdedgo modelu
  most_frequent_node_dict = {}

  # slownik w formacie: synset_id: synset_str
  synset_dict = {}

  for node, model_object in best_nodes_list:
    graph = model_object._g
    synset_id = graph.get_node_attribute("synset", node).synset_id

    POS = get_pos_int(graph, node)

    if model_object is model_object_NOUN:
      synset_id_NOUN = synset_id
    elif model_object is model_object_VERB:
      synset_id_VERB = synset_id

    if synset_id not in most_frequent_node_dict:
      most_frequent_node_dict[synset_id] = 0
    most_frequent_node_dict[synset_id] += 1

    if synset_id not in synset_dict:
      synset_str = make_unitstr(graph, node)
      synset_dict[synset_id] = synset_str

  synset_rank = sorted(most_frequent_node_dict.items(), key=operator.itemgetter(1), reverse=True)
  win_synset_id = synset_rank[0][0]
  win_synset_str = None

  # heurystyka: jesli 2 pierwsze synsety w synset_rank maja ta sama
  #             liczbe wystapien jako synset zwyciezca brany jest synset
  #             nalezacy do modelu dla odpowiedniej czesci mowy
  if len(synset_rank) > 1:
    if (POS == 1 or POS == 2) and synset_rank[0][1] == synset_rank[1][1]:
      win_synset_id = synset_id_VERB if POS == 1 else synset_id_NOUN

  win_synset_str = synset_dict[win_synset_id]

  return win_synset_id, win_synset_str, synset_rank

def make_unitstr(graph, node):
  fst_unitsstr = ''
  for lu in graph.get_node_attribute("synset", node).lu_set:
    lu_lemma = str(lu.lemma).replace(' ', '_')
    lu_variant = int(lu.variant)
    lu_domain = int(lu.domain)
    lu_domain_str = dom.DOMAINS[lu_domain] \
      if dom.DOMAINS.has_key(lu_domain) \
      else str(lu_domain)
    manual_synset_str = ('%s.%d(%d:%s) ' % \
                        (lu_lemma, 
                         lu_variant, 
                         lu_domain, 
                         lu_domain_str))
    fst_unitsstr += manual_synset_str
  return fst_unitsstr

def get_pos_int(graph, node):
  return next(iter(graph.get_node_attribute("synset", node).lu_set)).pos

def get_synset_rank_str(synset_rank):
  return ' '.join(['{0}/{1}'.format(synset_id, num_of_occur) \
                   for (synset_id, num_of_occur) in synset_rank])

def add_annotations_from_rank(token, synset_id, synset_str, synset_rank):
  md_ptr = token.get_metadata()
  if not md_ptr:
    token.create_metadata()
    md_ptr = token.get_metadata()

  md_ptr.set_attribute('sense:ukb:syns_id', str(synset_id))
  md_ptr.set_attribute('sense:ukb:syns_rank', get_synset_rank_str(synset_rank))
  md_ptr.set_attribute('sense:ukb:unitsstr', synset_str[:-1])

def write_ccl_file(cfg, ccl_document, outccl):
  writer = (
    corpus2.TokenWriter.create_stdout_writer('ccl', cfg.resources().tagset())
      if outccl[0] == '-' \
      else \
        corpus2.TokenWriter.create_path_writer(\
          'ccl', outccl, cfg.resources().tagset()))
  for paragraph in ccl_document.paragraphs():
    writer.write_chunk(paragraph)

def run_wosedon_multiple_models_classifier(config_files, models_dir, fpaths, verbose):
  models_objects_list = []
  for i, cfg in enumerate(config_files):
    print i
    print 'a'
    wsdlg = WoSeDonPlugin(cfg, models_dir[i] if models_dir else None, verbose)
    print 'b'
    models_objects_list.append(wsdlg)
    print 'c'

  for inccl in fpaths:
    if verbose:
      print >> sys.stderr, 'Running WoSeDoN for file:', inccl

    jobs = []
    threads_result_dict = {}
    for model_object in models_objects_list:
      threads_result_dict[model_object] = []
      thread = threading.Thread(target=model_object.run_wosedon(
                                  inccl, None, None, 
                                  threads_result_dict[model_object]))
      jobs.append(thread)
    for j in jobs:
      j.start()
    for j in jobs:
      j.join()

    models_objects = threads_result_dict.keys()
    first_model_object = models_objects[0]
    for i, ctx_rank_list in enumerate(threads_result_dict[first_model_object]):
      for j, (token, node_rank) in enumerate(ctx_rank_list):
        if node_rank:
          best_node = node_rank[0][0]
          best_nodes_list = [(best_node, first_model_object)]
          for model_object in models_objects[1:]:
            best_node = threads_result_dict[model_object][i][j][1][0][0]
            best_nodes_list.append((best_node, model_object))
          (win_synset_id, win_synset_str, synset_rank) = vote(best_nodes_list, \
                                                              models_objects_list[0], \
                                                              models_objects_list[1])
          add_annotations_from_rank(token, win_synset_id, win_synset_str, synset_rank)

    cfg = first_model_object._wosedon_cfg
    context = first_model_object._context
    out_ccl_filename = inccl.strip() + ".wosedon.xml"
    write_ccl_file(cfg, context.ccl_document(), out_ccl_filename)


class DifferentConfigAndModel(Exception):
  def __init__(self, message):
    self.message = message
  def __str__(self):
    return repr(self.message)


def main(argv = None):
  p = make_parser()
  args = p.parse_args(argv)

  config_files = get_files_paths_list(args.config_files)
  models_dir = get_files_paths_list(args.models_dir) if args.models_dir else []

  try:
    if args.models_dir and not len(config_files) == len(models_dir):
      raise DifferentConfigAndModel('Liczba podanych na wejciu plikow ' \
                                    'konfiguracyjnych jest ' \
                                    'rozna od liczby plikow z modelami!')
  except DifferentConfigAndModel as e:
    exit(e.message)

  fpaths = get_fpaths_list(args.ccl)

  run_wosedon_multiple_models_classifier(config_files, models_dir, fpaths, args.verbose)


if __name__ == '__main__':
  main()



#! /usr/bin/python
# *-* coding: utf-8 *-*

import codecs 
import requests
import json
import time
import os

from subprocess import Popen, PIPE



class Concraft(object):
    def __init__(self, model_name=None, model_path=None, concraft_path='./', port=3000, core=5):
        self.path = os.path.join(model_path, model_name)
        self.concraftpath = os.path.join(concraft_path, 'concraft-dag2-pl')
        self.port = port
	self.core = core
        self.concraftserver = Popen([self.concraftpath, 'server', '--port=%d' % port, '-i', self.path, '+RTS', '-N%d' % core],
                              stdin=PIPE, stdout=PIPE, stderr=PIPE)
        print(u"Concraft model " + model_name + u" loading…")
        loaded = False
        while not loaded:
            try:
                request_data = {'dag':''}
                r = requests.post('http://localhost:%d/parse' % port, data=json.dumps(request_data))
                loaded = True
                #print(u"loaded!")
            except requests.ConnectionError as e:
                #print(u"loading…")
                time.sleep(2)
   
    def tag(self, dag):
        json_string = ''
        for item in dag:
            num1, num2, (forma, lemat, tag, posp, kwal) = item
            line_string = '\t'.join((str(num1), str(num2), forma, lemat, tag, ','.join(posp), ','.join(kwal), '0.000', '', '' + '\n'))
            json_string = json_string + line_string
        analyse_list = []
        if json_string != '':
            request_data = {'dag':json_string + '\n'}
            r = requests.post('http://localhost:%d/parse' % self.port, data=json.dumps(request_data))                          
            for line in r.json()['dag'].split('\n'):
                if line != '':
                    num1, num2, forma, lemat, tag, posp, kwal, prob, interp, eos, disamb = line.strip('\n').split('\t')
                    eos = 'eos' if eos else None
                    disamb = 'disamb' if disamb else None
                    posp = posp.split(',') if posp else []
                    kwal = kwal.split(',') if kwal else []
                    interp = ''
                    analyse_list.append((int(num1), int(num2), (forma, lemat, tag, posp, kwal), prob, eos, disamb))
                else:
                    continue
        return analyse_list
   
    def done(self):
        self.concraftserver.terminate()




import unittest
from wosedon.reranks import NodeDegreeRanker
from wosedon.basegraph import BaseGraph
from wosedon.utils.config import Config



class TestNodeDegreeRanker(unittest.TestCase):
  
  def setUp(self):
    cfg = Config()
    cfg.parse('tests/#Data/wosedon_cosine.ini')
    cfg.resources().set_tagset('nkjp')
    cfg._context = 'Document'
    ctx = cfg.context('tests/#Data/little_ccl.xml', 'tests/#Data/little_ccl.xml')
    
    g = BaseGraph()
    g.init_graph()
    self.nodes = []
    for i in xrange(10):
      self.nodes.append(g.add_node(i))
    
    self.tokens = ctx.next().tokens()
    self.graph = g
    self.rerank = NodeDegreeRanker()
    self.options = cfg.rerank_options()
  
  
  def test_it(self):
    ranking = [
      (self.tokens[0], [(self.nodes[0], 3), (self.nodes[1], 2), (self.nodes[2], 1)]),
      (self.tokens[1], [(self.nodes[3], 5)]),
      (self.tokens[2], [(self.nodes[4], 11), (self.nodes[5], 7)]),
      (self.tokens[3], [(self.nodes[6], 13)]),
      (self.tokens[4], [(self.nodes[7], 31), (self.nodes[8], 23), (self.nodes[9], 17)])
    ]
    
    self.assertEquals(self.rerank.rerank(ranking, self.graph, self.options),
      [
        (self.tokens[0], [(self.nodes[0], 0), (self.nodes[1], 0), (self.nodes[2], 0)]),
        (self.tokens[1], [(self.nodes[3], 0)]),
        (self.tokens[2], [(self.nodes[4], 0), (self.nodes[5], 0)]),
        (self.tokens[3], [(self.nodes[6], 0)]),
        (self.tokens[4], [(self.nodes[7], 0), (self.nodes[8], 0), (self.nodes[9], 0)])
      ])
    
    self.graph.add_edge(0, 1)
    self.graph.add_edge(1, 2)
    
    self.graph.add_edge(3, 4)
    self.graph.add_edge(3, 5)
    self.graph.add_edge(3, 6)
    
    new_rank = self.rerank.rerank(ranking, self.graph, self.options)
    
    self.assertEquals(len(new_rank), 5)
    for i, (tok, rank) in enumerate(new_rank):
      self.assertIs(tok, self.tokens[i])
    
    self.assertEquals(new_rank[0][1][0][0], self.nodes[1])
    self.assertEquals(new_rank[1][1][0][0], self.nodes[3])
    self.assertEquals(new_rank[2][1][0][0], self.nodes[4])
    self.assertEquals(new_rank[3][1][0][0], self.nodes[6])
    self.assertEquals(new_rank[4][1][0][0], self.nodes[7])
    






#
# <one line to give the library's name and an idea of what it does.>
# Copyright (C) 2014  <copyright holder> <email>
# 
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
# 
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# 
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
# 
#

from graph_tool.centrality import pagerank
from wosedon.algorithms.wsdalgorithminterface import WSDAlgorithmInterface
from wosedon.ranking.wsd_ranking import WSDRanking

class GTPersonalizedW2WPRNorm(WSDAlgorithmInterface):
  """!
  Personalized Word-to-Word Page Rank algorithm based on the set of synsets 
  which correspond to lemmas from the analyzed context. Values in personalized
  vector "v" which corespond to all synsets which belong to lemmas from 
  the analyzed context depends on the number of lemmas meanings. Each word
  (lemma) from analyzed context is separately disambiguated. 

  Synsets which belong to word (lemma) which is actually disambiguated
  have value equal to 0 in personalized vector "v". While rest of synsets which
  correspond to words (lemmas) from the analyzed context have value
  calculated as describe below.

  If lemma has N meanings (synsets) then values in personalized vector 
  "v" for all its meanings (synsets) will be 1/N. If this same synset
  occur in two lemmas from the analyzed context then boths values are added.

  For example:

  Suppose that lemma "kot" is actually disambiguated.

  \code
  Lemma    Set of synsets for lemma
  kot      1, 2, 3, 4, 5
  pies     5, 6, 7

  Personalized vector "v":
  Synset ID             Value
  1 (kot-1)             0
  2 (kot-2)             0
  3 (kot-3)             0
  4 (kot-4)             0
  5 (kot-5) & (pies-1)  0 + 1/3
  6 (pies-2)            1/3
  7 (pies-3)            1/3
  \endcode

  Remaining synsets which do not belong to lemmas from the analyzed context
  have value equal to 0 in personalized vector "v".
  """

  def __init__(self, str_name = 'GTPersonalizedW2WPRNorm'):
    """!
    @param str_name - default value same as the class name "GTPersonalizedW2WPRNorm"
    """
    super(GTPersonalizedW2WPRNorm, self).__init__(str_name)
    self._context_node_dict = None

  def prepare_v(self, wsd_context, graph):
    """!
    Create personalized vector "v".

    @param wsd_context - object of WSDContext class
    @param graph - object of BaseGraph class
    @return PropertyMap (see: http://graph-tool.skewed.de/static/doc/graph_tool.html?highlight=new_vertex_property#graph_tool.Graph.new_vertex_property) of personalized vector "v"
    """
    
    u = graph.use_graph_tool().new_vertex_property("double")
    for node in self._context_node_dict:  # all others have 0.0 by default
      u[node.use_graph_tool()] = self._multiply_factor * self._context_node_dict[node]
    
    return u

  def run(self, wsd_context, graph, options, resources):
    """!
    Disambiguate analyzed context.

    @param wsd_context - object of WSDContext class
    @param graph - object of BaseGraph class
    @param options - object of AlgorithmOptions class
    @param resources - object of Resources class
    @return tuple of object of WSDRanking class and number of algorithm iterations
    """
    ret_iter_list = []

    wsd_rank = WSDRanking()
    (lemma_on_node_dict, lemma_on_only_synset_node_dict) = \
      wsd_rank.get_lemma_on_node_dict(wsd_context, graph, options.ini_nodes())

    for analyzed_lemma, nodes in lemma_on_node_dict.iteritems():
      self._set_context_node_dict(analyzed_lemma, lemma_on_node_dict)

      pers_v = self.prepare_v(wsd_context, graph)
      (ranking, ret_iter) = pagerank(graph.use_graph_tool(), 
                                     pers = pers_v, 
                                     max_iter = 2 * options.max_iter(),
                                     damping = options.damping_factor(),
                                     ret_iter = True,
                                     weight = graph.use_graph_tool().ep["weight"])

      ret_iter_list.append(ret_iter)
      ranking = graph.ungraph_tool(ranking)
      wsd_rank.set_ranking_for_lemma(analyzed_lemma[0], 
                                     analyzed_lemma[1], 
                                     ranking, 
                                     create_new = False)

    return (wsd_rank, float(sum(ret_iter_list)) / float(len(ret_iter_list)))

  def _set_context_node_dict(self, analyzed_lemma, lemma_on_node_dict):
    """!
    Create a dictionary where key is object of node from the graph which
    corespond to the synset which belong to lemma from the analyzed context,
    excluding synsets which belong to disambiguated word (lemma).
    Whereas value is a value which synset from the key will be have in
    personalized vector "v".

    @param analyzed_lemma - lemma which is actually disambiguated
    @param lemma_on_node_dict - dictionary where key is (lemma, pos_str) and
                                value is set of synsets (nodes from the graph)
                                belong to this lemma
    """
    self._context_node_dict = {}
    for lemma, nodes in lemma_on_node_dict.iteritems():
      if lemma == analyzed_lemma:
        continue
      synset_set_size = float(len(nodes))
      for node in nodes:
        if self._context_node_dict.has_key(node):
          self._context_node_dict[node] += 1.0 / synset_set_size
        else:
          self._context_node_dict[node] = 1.0 / synset_set_size



# -*- coding: utf-8 -*-

import unittest
from wosedon.utils.resources import Resources
from wosedon.utils.buildoptions import BuildOptions
from wosedon.gbuilders.lexicalunitgraphbuilder import LexicalUnitGraphBuilder



def leniter(i):
  """ leniterght of any iterable """
  return sum(1 for e in i)


class TestLexicalUnitGB_DirectnessAndDuplicates(unittest.TestCase):
  
  def setUp(self):
    self.res = Resources()
    self.res.set_plwn_graph_file('tests/#Data/plwnik')
    self.opts = BuildOptions()
  
  
  def test_undirected_not_unique(self):
    self.opts.set_directed_graph(False)
    self.opts.set_unique_edges(False)
    
    g = LexicalUnitGraphBuilder(self.res, self.opts).build_graph()
    
    self.assertEquals(leniter(g.get_edges_between(3, 6)), 2)
    self.assertEquals(leniter(g.get_edges_between(5, 8)), 2)
    self.assertEquals(leniter(g.get_edges_between(0, 10)), 2)
    self.assertEquals(leniter(g.get_edges_between(5, 1)), 1)
    self.assertEquals(leniter(g.get_edges_between(1, 5)), 1)
  
  
  def test_directed_not_unique(self):
    self.opts.set_directed_graph(True)
    self.opts.set_unique_edges(False)
    
    g = LexicalUnitGraphBuilder(self.res, self.opts).build_graph()
    
    self.assertEquals(leniter(g.get_edges_between(3, 6)), 2)
    self.assertEquals(leniter(g.get_edges_between(8, 5)), 2)
    self.assertEquals(leniter(g.get_edges_between(0, 10)), 1)
    self.assertEquals(leniter(g.get_edges_between(5, 1)), 1)
    self.assertEquals(leniter(g.get_edges_between(1, 5)), 0)
    self.assertEquals(leniter(g.get_edges_between(7, 7)), 1)
  
  
  def test_undirected_unique(self):
    self.opts.set_directed_graph(False)
    self.opts.set_unique_edges(True)
    
    g = LexicalUnitGraphBuilder(self.res, self.opts).build_graph()
    
    for s in g.all_nodes():
      for t in g.all_nodes():
        if s != t:
          self.assertLessEqual(leniter(g.get_edges_between(s, t)), 1)
  
  
  def test_directed_unique(self):
    self.opts.set_directed_graph(True)
    self.opts.set_unique_edges(True)
    
    g = LexicalUnitGraphBuilder(self.res, self.opts).build_graph()
    
    for s in g.all_nodes():
      for t in g.all_nodes():
        self.assertLessEqual(leniter(g.get_edges_between(s, t)), 1)




class TestLexicalUnitGB_DirectnessAndDuplicatesWithReverseEdges(unittest.TestCase):
  
  def setUp(self):
    self.res = Resources()
    self.res.set_plwn_graph_file('tests/#Data/plwnik')
    self.opts = BuildOptions()
    self.opts.set_add_reversed_edges({11:1011})
  
  
  def test_undirected_not_unique(self):
    self.opts.set_directed_graph(False)
    self.opts.set_unique_edges(False)
    
    g = LexicalUnitGraphBuilder(self.res, self.opts).build_graph()
    
    self.assertEquals(leniter(g.get_edges_between(1, 3)), 2)
    self.assertEquals(leniter(g.get_edges_between(5, 8)), 3)
  
  
  def test_directed_not_unique(self):
    self.opts.set_directed_graph(True)
    self.opts.set_unique_edges(False)
    
    g = LexicalUnitGraphBuilder(self.res, self.opts).build_graph()
    
    self.assertEquals(leniter(g.get_edges_between(3, 1)), 1)
    self.assertEquals(leniter(g.get_edges_between(5, 8)), 1)
  
  
  def test_undirected_unique(self):
    self.opts.set_directed_graph(False)
    self.opts.set_unique_edges(True)
    
    g = LexicalUnitGraphBuilder(self.res, self.opts).build_graph()
    
    for s in g.all_nodes():
      for t in g.all_nodes():
        if s != t:
          self.assertLessEqual(leniter(g.get_edges_between(s, t)), 1)
  
  
  def test_directed_unique(self):
    self.opts.set_directed_graph(True)
    self.opts.set_unique_edges(True)
    
    g = LexicalUnitGraphBuilder(self.res, self.opts).build_graph()
    
    for s in g.all_nodes():
      for t in g.all_nodes():
        self.assertLessEqual(leniter(g.get_edges_between(s, t)), 1)
        



class TestLexicalUnitGB(unittest.TestCase):
  
  def setUp(self):
    self.res = Resources()
    self.res.set_plwn_graph_file('tests/#Data/plwnik')
    self.opts = BuildOptions()
    self.opts.set_directed_graph(True)
    self.opts.set_unique_edges(False)
    self.opts.set_add_reversed_edges({11:1011, 13:13, 12:14})
  
  
  def test_syn_rel_ids(self):
    self.opts.set_lu_rel_ids(set([11, 15]))
    
    g = LexicalUnitGraphBuilder(self.res, self.opts).build_graph()
    
    for e in g.all_edges():
      self.assertIn(e.rel_id, [11, 15])
  
  
  #TODO test_accept_lexicon
  
  
  def test_accept_pos(self):
    self.opts.set_accept_pos(set([1,3]))
    
    g = LexicalUnitGraphBuilder(self.res, self.opts).build_graph()
    
    for n in g.all_nodes():
      self.assertIn(n.lu.pos, [1,3])
  
  
  def test_add_reversed_edges(self):
    g = LexicalUnitGraphBuilder(self.res, self.opts).build_graph()
    
    for e in g.all_edges():
      if e.rel_id == 11:
        self.assertIn(1011, [k.rel_id for k in g.get_edges_between(e.target(), e.source())])
      elif e.rel_id == 13:
        self.assertIn(13, [k.rel_id for k in g.get_edges_between(e.target(), e.source())])
      elif e.rel_id == 12:
        self.assertIn(14, [k.rel_id for k in g.get_edges_between( e.target(), e.source())])
  
  
  def test_edge_names(self):
    g = LexicalUnitGraphBuilder(self.res, self.opts).build_graph()
    
    for e in g.all_edges():
      self.assertRegexpMatches(e.rel, r'l\d+')






























#
# <one line to give the library's name and an idea of what it does.>
# Copyright (C) 2014  <copyright holder> <email>
# 
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
# 
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# 
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
# 
#

from context import Context, WSDContext

class Document(Context):
  def __init__(self, tagset, ccl_doc, ccl_rel = None):
    super(Document, self).__init__(tagset, ccl_doc, ccl_rel, 'Document')
    self.reset()
  
  def reset(self):
    self._read = False
  
  def next(self):
    if self._read:
      return None
    toks = []
    for p in self._ccl_doc.paragraphs():
      for s in p.sentences():
        toks.extend(s.tokens())
    self._read = True
    return WSDContext(tagset = self._tagset,
                      tokens = toks)



#
# <one line to give the library's name and an idea of what it does.>
# Copyright (C) 2014  <copyright holder> <email>
# 
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
# 
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# 
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
# 
#

from collections import defaultdict
import logging
import sys

class WSDRanking():
  def __init__(self):
    self._ranking_for_lemma_dict = {}
    self._ctx_ranking = None
    self._lemma_on_node_dict = None
    self._lemma_on_only_synset_node_dict = None


  def set_ranking_for_context(self, wsd_context, graph):
    """
    Sets internal variable _ctx_ranking. It will become a list of tokens, each
    of which will be assigned the ranking of its lemma. It will be new, pure
    ranking, free off unrelated nodes (cf. set_ranking_for_lemma()).
    """
    lon, lon_only_synset = self.get_lemma_on_node_dict(wsd_context, graph)

    rank_for_context_dict = defaultdict(list)
    for (lemma, pos_str), nodes in lon_only_synset.iteritems():
      ranking = self.get_ranking_for_lemma(lemma, pos_str)
      if not ranking:
        continue
      for node in nodes:
        rank_for_context_dict[(lemma, pos_str)].append((node, ranking[node]))
      all_rank = rank_for_context_dict[(lemma, pos_str)]
      sorted_all_rank = sorted(all_rank, key = lambda rank: rank[1], reverse = True)
      rank_for_context_dict[(lemma, pos_str)] = sorted_all_rank

    rank_for_tokens_list = []
    for t in wsd_context.tokens():
      lemma = wsd_context.get_token_lemma_str(t).lower()
      #lemma = wsd_context.get_token_lemma_str(t)
      pos_str = wsd_context.get_token_coarse_pos(t)
      if rank_for_context_dict.has_key((lemma, pos_str)):
        rank = rank_for_context_dict[(lemma, pos_str)]
        rank_for_tokens_list.append((t, rank))
      else:
        rank_for_tokens_list.append((t, None))

    self._ctx_ranking = rank_for_tokens_list

  def get_ranking_for_context(self, wsd_context, graph):
    """
    Returns a list [(token, its ranking) for each token in context].
    """
    self.set_ranking_for_context(wsd_context, graph)
    return self._ctx_ranking


  def set_ranking_for_lemma(self, lemma, pos_str, ranking, create_new = False):
    """
    Sets a ranking that will be used for that lemma. Ranking is a mapping from
    nodes to floats. Such ranking may contain nodes absolutely unconnected with
    the lemma, WSDRanking is able to ignore them.
    """
    if create_new:
      self._ranking_for_lemma_dict[(lemma, pos_str)] = ranking.copy()
    else:
      self._ranking_for_lemma_dict[(lemma, pos_str)] = ranking

  def get_ranking_for_lemma(self, lemma, pos_str):
    """
    Returns previoulsly set ranking for the lemma.
    """
    ranking = None
    if self._ranking_for_lemma_dict.has_key((lemma, pos_str)):
      ranking = self._ranking_for_lemma_dict[(lemma, pos_str)]
    else:
      logging.getLogger(__name__).warning('There is no ranking for lemma "%s".', str(lemma))
    return ranking


  def _get_context_lemma_set(self, wsd_context):
    context_lemma_set = set()
    for t in wsd_context.tokens():
      lemma = wsd_context.get_token_lemma_str(t).lower()
      #lemma = wsd_context.get_token_lemma_str(t)
      pos_str = wsd_context.get_token_coarse_pos(t)
      context_lemma_set.add((lemma, pos_str))
    return context_lemma_set

  def _set_lemma_on_node_dict(self, wsd_context, graph, ini_nodes=None):
    context_lemma_set = self._get_context_lemma_set(wsd_context)
    self._lemma_on_node_dict = defaultdict(set)
    self._lemma_on_only_synset_node_dict = defaultdict(set)
    if not ini_nodes:
      for node in graph.all_nodes():
        if node.synset:
          self._set_synset_node(graph, 
                                node, 
                                wsd_context, 
                                context_lemma_set)
    else:
      for node in graph.all_nodes():
        if node.synset:
          self._set_synset_node(graph, 
                                node, 
                                wsd_context, 
                                context_lemma_set, 
                                'synset' in ini_nodes)
        elif 'sumo' in ini_nodes and node.sumo:
          self._set_not_synset_node(graph, 
                                    node, 
                                    wsd_context, 
                                    context_lemma_set)
        elif 'wnd' in ini_nodes and node.wnd:
          self._set_not_synset_node(graph, 
                                    node, 
                                    wsd_context, 
                                    context_lemma_set)
        elif 'msr' in ini_nodes and node.msr:
          self._set_not_synset_node(graph, 
                                    node, 
                                    wsd_context, 
                                    context_lemma_set)

  def _set_synset_node(self, graph, node, wsd_context, context_lemma_set, 
                       synset_in_ini_nodes=True):
    lemma_and_pos_str_set = self._get_lemma_and_pos_str_set(graph, 
                                                            node, 
                                                            wsd_context, 
                                                            context_lemma_set)
    if lemma_and_pos_str_set:
      for (lemma, pos_str) in lemma_and_pos_str_set:
        if synset_in_ini_nodes:
          self._lemma_on_node_dict[(lemma, pos_str)].add(node)
        self._lemma_on_only_synset_node_dict[(lemma, pos_str)].add(node)

  def _set_not_synset_node(self, graph, node, wsd_context, context_lemma_set):
    for ngb in node.all_neighbours():
      if ngb.synset:
        lemma_and_pos_str_set = self._get_lemma_and_pos_str_set(graph, 
                                                                ngb, 
                                                                wsd_context, 
                                                                context_lemma_set)
        if lemma_and_pos_str_set:
          for (lemma, pos_str) in lemma_and_pos_str_set:
            self._lemma_on_node_dict[(lemma, pos_str)].add(node)

  def _get_lemma_and_pos_str_set(self, graph, node, wsd_context, context_lemma_set):
    lemma_and_pos_str_set = set()
    lu_set = node.synset.lu_set
    for lu in lu_set:
      lemma = lu.lemma.lower()
      #lemma = lu.lemma
      pos_str = wsd_context.convert_num_pos_to_coarse_str(lu.pos)
      if (lemma, pos_str) in context_lemma_set:
        lemma_and_pos_str_set.add((lemma, pos_str))
    return lemma_and_pos_str_set

  def get_lemma_on_node_dict(self, wsd_context, graph, ini_nodes=None):
    """
    Returns two dictionaries. Both map each lemma from context to corresponding
    nodes.
    
    If ini_nodes is not set, both will acknowledge all synset nodes and only them.
    Otherwise in the first dictionary there will be nodes of types listed in ini_nodes,
    and only they. Whereas the second dict will use synset nodes and nothing more.
    
    """
    if not self._lemma_on_node_dict:
      self._set_lemma_on_node_dict(wsd_context, graph, ini_nodes)
    return self._lemma_on_node_dict, self._lemma_on_only_synset_node_dict



#
# <one line to give the library's name and an idea of what it does.>
# Copyright (C) 2014  <copyright holder> <email>
# 
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
# 
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# 
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
# 
#

import corpus2
from resources import Resources
from buildoptions import BuildOptions
from mergeoptions import MergeOptions
from algorithmoptions import AlgorithmOptions
from rerankoptions import RerankOptions

MODULE_GBUILDERS = 'wosedon.gbuilders'
MODULE_MERGERS = 'wosedon.mergers'
MODULE_CONTEXT = 'wosedon.context'
MODULE_ALGORITHMS = 'wosedon.algorithms'
MODULE_RERANKS = 'wosedon.reranks'

SEC_WSD = 'wosedon'
WSD_OPT_GBUILDERS = 'gbuilders'
WSD_OPT_CONTEXT = 'context'
WSD_OPT_MERGERS = 'mergers'
WSD_OPT_WSD_ALG = 'wsdalgorithm'
WSD_OPT_RERANKERS = 'rerankers'
WSD_OPT_USE_WEIGHTS = 'use_weights'
WSD_OPT_WEIGHTS = 'weights'

SEC_WSD_RES = 'wosedon:resources'
WSD_RES_OPT_SUMO_GRAPH = 'sumo_graph_file'
WSD_RES_OPT_CCL_GRAPH = 'ccl_graph_file'
WSD_RES_OPT_SUMO_MAPPING = 'mapping_sumo_file'
WSD_RES_OPT_WND_GRAPH = 'wnd_graph_file'
WSD_RES_OPT_WND_MAPPING = 'mapping_wnd_file'
WSD_RES_OPT_MSR = 'msr_file'
WSD_RES_OPT_PLWN_GRAPH = 'plwn_graph_file'
WSD_RES_OPT_GLOSS_FILE = 'gloss_file'
WSD_RES_OPT_GLOSS_REL_FILE = 'gloss_rel_file'
WSD_RES_OPT_TAGSET = 'tagset'
WSD_RES_OPT_IMPEDANCE_FILE = 'impedance_table'

SEC_WSD_BUILD = 'wosedon:build_options'
WSD_BUILD_OPT_ACC_LEXICON = 'accept_lexicon'
WSD_BUILD_OPT_UNIQ_EDGES = 'unique_edges'
WSD_BUILD_OPT_DI_GRAPH = 'directed_graphs'
WSD_BUILD_OPT_SYN_RELS_IDS = 'syn_rel_ids'
WSD_BUILD_OPT_LU_RELS_IDS = 'lu_rel_ids'
WSD_BUILD_OPT_ACC_POS = 'accept_pos'
WSD_BUILD_OPT_ADD_REVERSED_EDGES = 'add_reversed_edges'

SEC_WSD_MERGE = 'wosedon:merge_options'

SEC_WSD_ALG = 'wosedon:wsd_alg'
WSD_ALG_DAMP_FACTOR = 'damping_factor'
WSD_ALG_MAX_ITER = 'max_iter'
WSD_ALG_INI_NODES = 'ini_nodes'
WSD_ALG_LESK_FUNC = 'lesk_function'
WSD_ALG_LESK_FILT = 'lesk_filter'

SEC_WSD_RERANK = 'wosedon:rerank_options'
WSD_RERANK_PERCENTAGE_DIFF = 'percentage_diff'

SEC_WSD_LESK_FILTER = 'wosedon:lesk_filter'
WSD_LF_OPT_INPUT_FILE = 'list_file'
WSD_LF_OPT_ALLOW_ONLY = 'allow_only'


class Config:
  def __init__(self, cfg_file_path=''):
    from ConfigParser import ConfigParser
    self._config = ConfigParser()
    
    # list of graph builders
    self._gbuilders = []
    # list of mergers
    self._mergers = []
    # algorithm which will be used into wsd process
    self._wsd_algorithm = None
    # list of rerankers used after running the algorithm
    self._rerankers = []

    # Context type
    self._context = None
    # Should weights be used
    self._use_weights = False
    # Weights
    self._weights = {}

    # Resources object
    self._resources = None
    # BuildOptions object
    self._build_options = None
    # MergeOptions object
    self._merge_options = None
    # AlgorithmOptions object
    self._algorithm_options = None
    # RerankOptions object
    self._rerank_options = None

    self._cfg_file_path = cfg_file_path
    if cfg_file_path:
      self.parse(cfg_file_path)

  def gbuilders(self):
    gb_list = []
    for gb in self._gbuilders:
      module = __import__(MODULE_GBUILDERS, fromlist=[gb])
      gb_list.append(getattr(module, gb)(self.resources(), self.build_options()))
    return gb_list

  def mergers(self):
    mrg_list = []
    for mrg in self._mergers:
      module = __import__(MODULE_MERGERS, fromlist=[mrg])
      mrg_list.append(getattr(module, mrg)(self.resources(), self.merge_options()))
    return mrg_list
  
  def wsd_algorithm(self):
    if isinstance(self._wsd_algorithm, str):
      module = __import__(MODULE_ALGORITHMS, fromlist=[self._wsd_algorithm])
      self._wsd_algorithm = getattr(module, self._wsd_algorithm)()
    return self._wsd_algorithm
  
  def rerankers(self):
    if isinstance(self._rerankers, str):
      reranks = []
      for rerank in self._rerankers.split():
        module = __import__(MODULE_RERANKS, fromlist=[rerank])
        reranks.append(getattr(module, rerank)())
      self._rerankers = reranks
    return self._rerankers
  
  def context(self, ccl_file, ccl_rel):
    if self._context:
      cclreader = corpus2.CclRelReader(self.resources().tagset(), 
                                       ccl_file, ccl_rel)
      ccldoc = cclreader.read()
      module = __import__(MODULE_CONTEXT, fromlist=[self._context])
      return getattr(module, self._context)(self.resources().tagset(), ccldoc, ccldoc.relations())
    return None

  def create_context(self, document):
    if self._context:
      module = __import__(MODULE_CONTEXT, fromlist=[self._context])
      return getattr(module, self._context)(
                                            self.resources().tagset(), 
                                            document, 
                                            document.relations())
    else:
      return None
  
  def weights(self):
    if self._use_weights:
      return self._weights
    return None

  def resources(self):
    return self._resources
  
  def build_options(self):
    return self._build_options

  def merge_options(self):
    return self._merge_options
  
  def algorithm_options(self):
    return self._algorithm_options

  def rerank_options(self):
    return self._rerank_options

  def parse(self, cfg_file_path):
    self._cfg_file_path = cfg_file_path
    self._resources = Resources()
    self._build_options = BuildOptions()
    self._merge_options = MergeOptions()
    self._algorithm_options = AlgorithmOptions()
    self._rerank_options = RerankOptions()
    
    with open(self._cfg_file_path, 'r') as ini_file:
      self._config.readfp(ini_file)

      # Obligatory option
      # gbuilders / makes string-like representation only...
      self._gbuilders = \
          self._config.get(SEC_WSD, WSD_OPT_GBUILDERS).split()

      # Obligatory option
      # A string like representation of a context type
      self._context = self._config.get(SEC_WSD, WSD_OPT_CONTEXT)

      # Mergers are optional, but the option is in an obligatory section.
      if self._config.has_option(SEC_WSD, WSD_OPT_MERGERS):
        # mergers / makes string-like representation only...
        self._mergers = \
            self._config.get(SEC_WSD, WSD_OPT_MERGERS).split()

      # Obligatory option
      # disambiguation method
      self._wsd_algorithm = self._config.get(SEC_WSD, WSD_OPT_WSD_ALG)

      # Rerankers are optional, but the option is in an obligatory section.
      if self._config.has_option(SEC_WSD, WSD_OPT_RERANKERS):
        # The split and class initializations are in method rerankers
        self._rerankers = self._config.get(SEC_WSD, WSD_OPT_RERANKERS)

      # Weights
      if self._config.has_option(SEC_WSD, WSD_OPT_USE_WEIGHTS):
        self._use_weights = self._config.getboolean(SEC_WSD, WSD_OPT_USE_WEIGHTS)
      if self._config.has_option(SEC_WSD, WSD_OPT_WEIGHTS):
        self._weights = self._build_dict(
                SEC_WSD,
                WSD_OPT_WEIGHTS,
                key_type = str,
                val_type = float
            )

      # resources
      self._load_resources()

      # build options
      self._load_build_options()

      # merge options
      self._load_merge_options()
      
      # algorithm options
      self._load_algorithm_options()

      # rerank options
      self._load_rerank_options()

  def _load_resources(self):
    """ Read and load all available resources from the config. """
    # This option is obligatory, ConfigParser will raise an exception if it
    # is missing.
    self._resources.set_plwn_graph_file(
        self._config.get(SEC_WSD_RES, WSD_RES_OPT_PLWN_GRAPH)
    )
    # This option is obligatory, ConfigParser will raise an exception if it
    # is missing.
    self._resources.set_tagset(self._config.get(SEC_WSD_RES, WSD_RES_OPT_TAGSET))
    
    # This section is obligatory, thus no need to check whether it exists
    # or not. An exception should be raised either way.
    if self._config.has_option(SEC_WSD_RES, WSD_RES_OPT_SUMO_GRAPH):
      self._resources.set_sumo_graph_file(
          self._config.get(SEC_WSD_RES, WSD_RES_OPT_SUMO_GRAPH)
      )
    if self._config.has_option(SEC_WSD_RES, WSD_RES_OPT_CCL_GRAPH):
      self._resources.set_ccl_graph_file(
          self._config.get(SEC_WSD_RES, WSD_RES_OPT_CCL_GRAPH)
      )
    if self._config.has_option(SEC_WSD_RES, WSD_RES_OPT_SUMO_MAPPING):
      self._resources.set_mapping_sumo_file(
          self._config.get(SEC_WSD_RES, WSD_RES_OPT_SUMO_MAPPING)
      )
    if self._config.has_option(SEC_WSD_RES, WSD_RES_OPT_WND_GRAPH):
      self._resources.set_wnd_graph_file(
          self._config.get(SEC_WSD_RES, WSD_RES_OPT_WND_GRAPH)
      )
    if self._config.has_option(SEC_WSD_RES, WSD_RES_OPT_WND_MAPPING):
      self._resources.set_mapping_wnd_file(
          self._config.get(SEC_WSD_RES, WSD_RES_OPT_WND_MAPPING)
      )
    if self._config.has_option(SEC_WSD_RES, WSD_RES_OPT_MSR):
      self._resources.set_msr_file(
          self._config.get(SEC_WSD_RES, WSD_RES_OPT_MSR)
      )
    if self._config.has_option(SEC_WSD_RES, WSD_RES_OPT_GLOSS_FILE):
      self._resources.set_gloss_file(
          self._config.get(SEC_WSD_RES, WSD_RES_OPT_GLOSS_FILE)
      )
    if self._config.has_option(SEC_WSD_RES, WSD_RES_OPT_GLOSS_REL_FILE):
      self._resources.set_gloss_rel_file(
          self._config.get(SEC_WSD_RES, WSD_RES_OPT_GLOSS_REL_FILE)
      )
    if self._config.has_option(SEC_WSD_RES, WSD_RES_OPT_IMPEDANCE_FILE):
      self._resources.set_impedance_table(
          self._config.get(SEC_WSD_RES, WSD_RES_OPT_IMPEDANCE_FILE)
      )

  def _load_build_options(self):
    """ Read and load all available build options from the config. """
    if self._config.has_section(SEC_WSD_BUILD):
      if self._config.has_option(SEC_WSD_BUILD, WSD_BUILD_OPT_ACC_LEXICON):
        self._build_options.set_accept_lexicon(
            [str(lexicon) for lexicon in
                self._config.get(
                    SEC_WSD_BUILD,
                    WSD_BUILD_OPT_ACC_LEXICON).split()]
        )
      if self._config.has_option(SEC_WSD_BUILD, WSD_BUILD_OPT_UNIQ_EDGES):
        self._build_options.set_unique_edges(
            self._config.getboolean(SEC_WSD_BUILD, WSD_BUILD_OPT_UNIQ_EDGES)
        )
      if self._config.has_option(SEC_WSD_BUILD, WSD_BUILD_OPT_DI_GRAPH):
        self._build_options.set_directed_graph(
            self._config.getboolean(SEC_WSD_BUILD, WSD_BUILD_OPT_DI_GRAPH)
        )
      if self._config.has_option(SEC_WSD_BUILD, WSD_BUILD_OPT_SYN_RELS_IDS):
        self._build_options.set_syn_rel_ids(
            [int(ids) for ids in
                self._config.get(
                    SEC_WSD_BUILD,
                    WSD_BUILD_OPT_SYN_RELS_IDS).split()]
        )
      if self._config.has_option(SEC_WSD_BUILD, WSD_BUILD_OPT_LU_RELS_IDS):
        self._build_options.set_lu_rel_ids(
            [int(ids) for ids in
                self._config.get(
                    SEC_WSD_BUILD,
                    WSD_BUILD_OPT_LU_RELS_IDS).split()]
        )
      if self._config.has_option(SEC_WSD_BUILD, WSD_BUILD_OPT_ACC_POS):
        self._build_options.set_accept_pos(
            [int(pos) for pos in
                self._config.get(
                    SEC_WSD_BUILD,
                    WSD_BUILD_OPT_ACC_POS).split()]
        )
      if self._config.has_option(SEC_WSD_BUILD, WSD_BUILD_OPT_ADD_REVERSED_EDGES):
        self._build_options.set_add_reversed_edges(
            self._build_dict(
                SEC_WSD_BUILD,
                WSD_BUILD_OPT_ADD_REVERSED_EDGES,
                key_type=int,
                val_type=int
            )
        )

  def _build_dict(self, section, option, key_type=str, val_type=str):
    """ Convert a dictionary string into a dictionary object.

    A dictionary string looks as follows:
      key1:val1 key2:val2 key3:val3 ...

    @param section the name of a section in the config file.
    @type section: str
    @param option the name of a section's option in the configure file.
    @type option: str
    @param key_type a function for key convertion, a key is a string.
    @type key_type: function
    @param val_type a function for value convertion, a value is a string.
    @type val_type: function

    @return a dictionary object
    @rtype: dict
    """
    raw_weights = self._config.get(section, option).split()
    weight_dict = {}
    for id_weight in raw_weights:
      id_, weight = id_weight.split(':')
      weight_dict[key_type(id_)] = val_type(weight)
    return weight_dict

  def _load_merge_options(self):
    """ Read and load all available merge options from the config. """
    if self._config.has_section(SEC_WSD_MERGE):
      pass

  def _load_algorithm_options(self):
    """ Read and load all available algorithm options from the config. """
    # This section is obligatory, thus no need to check whether it exists
    # or not. An exception should be raised either way.
    if self._config.has_option(SEC_WSD_ALG, WSD_ALG_DAMP_FACTOR):
      self._algorithm_options.set_damping_factor(
          self._config.get(SEC_WSD_ALG, WSD_ALG_DAMP_FACTOR)
      )
    if self._config.has_option(SEC_WSD_ALG, WSD_ALG_MAX_ITER):
      self._algorithm_options.set_max_iter(
          self._config.getint(SEC_WSD_ALG, WSD_ALG_MAX_ITER)
      )
    if self._config.has_option(SEC_WSD_ALG, WSD_ALG_INI_NODES):
      self._algorithm_options.set_ini_nodes(
        [ini_node for ini_node in
          self._config.get(
            SEC_WSD_ALG, 
            WSD_ALG_INI_NODES).split()]
      )      
    if self._config.has_option(SEC_WSD_ALG, WSD_ALG_LESK_FUNC):
      self._algorithm_options.set_lesk_function(
          self._config.get(SEC_WSD_ALG, WSD_ALG_LESK_FUNC)
      )
    if self._config.has_option(SEC_WSD_ALG, WSD_ALG_LESK_FILT):
      self._algorithm_options.set_lesk_filter(
          self._config.get(SEC_WSD_ALG, WSD_ALG_LESK_FILT)
      )
    
    if self._config.has_section(SEC_WSD_LESK_FILTER):
      lfo = self._algorithm_options.lesk_filter_opts()
      
      if self._config.has_option(SEC_WSD_LESK_FILTER, WSD_LF_OPT_INPUT_FILE):
        lfo.set_input_file(self._config.get(SEC_WSD_LESK_FILTER, WSD_LF_OPT_INPUT_FILE))
      
      if self._config.has_option(SEC_WSD_LESK_FILTER, WSD_LF_OPT_ALLOW_ONLY):
        lfo.set_allow_only(self._config.getboolean(SEC_WSD_LESK_FILTER, WSD_LF_OPT_ALLOW_ONLY))
    

  def _load_rerank_options(self):
    """ Read and load all available rerankers options from the config. """
    if self._config.has_option(SEC_WSD_RERANK, WSD_RERANK_PERCENTAGE_DIFF):
      self._rerank_options.set_percentage_diff(
          self._config.getint(SEC_WSD_RERANK, WSD_RERANK_PERCENTAGE_DIFF)
      )
    # This option is obligatory, ConfigParser will raise an exception if it
    # is missing.
    self._rerank_options.set_tagset(self._resources.tagset())



# -*- coding: utf-8 -*-

import unittest
from wosedon.utils.resources import Resources
from wosedon.utils.buildoptions import BuildOptions
from wosedon.gbuilders.msrgraphbuilder import MSRGraphBuilder





def leniter(i):
  """ Lenght of any iterable """
  return sum(1 for e in i)




class TestMSRGB(unittest.TestCase):
  
  def setUp(self):
    self.res = Resources()
    self.res.set_msr_file('tests/#Data/msr_graph')
    self.opts = BuildOptions()
    self.opts.set_unique_edges(False)
  
  
  def test_edge_names(self):
    g = MSRGraphBuilder(self.res, self.opts).build_graph()
    
    for e in g.all_edges():
      self.assertEquals(e.rel, '')
      self.assertIsInstance(e.weight, float)
  
  
  def test_numbers(self):
    g = MSRGraphBuilder(self.res, self.opts).build_graph()
    
    self.assertEquals(leniter(g.all_nodes()), 7)
    self.assertEquals(leniter(g.all_edges()), 11)
  
  
  def test_unique(self):
    self.opts.set_unique_edges(True)
    g = MSRGraphBuilder(self.res, self.opts).build_graph()
    
    for s in g.all_nodes():
      for t in g.all_nodes():
        self.assertLessEqual(leniter(g.get_edges_between(s, t)), 1)
    
    self.assertEquals(leniter(g.all_nodes()), 7)
    self.assertEquals(leniter(g.all_edges()), 8)






























import math

from wosedon.lesk_functions.dictionarybased import DictionaryBased


def _cosine_similarity(v1, v2):
  sumxx, sumxy, sumyy = 0, 0, 0
  for i in xrange(len(v1)):
    x = v1[i]; y = v2[i]
    sumxx += x*x
    sumyy += y*y
    sumxy += x*y
  if sumxx <= 0 or sumyy <= 0:
    return 0.0
  return sumxy/math.sqrt(sumxx*sumyy)



class Cosine(DictionaryBased):
  """! Vector similarity between definition and context. """
  
  def __init__(self, resources, filter_obj, graph, options, str_name='Cosine'):
    super(Cosine, self).__init__(resources, filter_obj, graph, options, str_name)
  
  def compare(self, definition_data, context_data):
    vec1 = []
    vec2 = []
    for (lemma, pos_str) in definition_data:
      vec1.append(definition_data[(lemma, pos_str)] / float(len(definition_data)))
      if (lemma, pos_str) in context_data:
        vec2.append(context_data[(lemma, pos_str)] / float(len(context_data)))
      else:
        vec2.append(0)
    
    return _cosine_similarity(vec1, vec2)
























# -*- coding: utf-8 -*-

import unittest
from wosedon.ranking.wsd_ranking import WSDRanking
from wosedon.basegraph import BaseGraph
from wosedon.utils.config import Config


@unittest.skip("Very mysterious errors, probably outside of python.")  #FIXME to powinno działać
class TestWSDRanking(unittest.TestCase):
  
  def setUp(self):
    class Synset:
      pass
      
    class LU:
      def __init__(self, lemma, pos):
        self.lemma = lemma
        self.pos = pos
    
    g = BaseGraph()
    g.init_graph()
    g.create_node_attribute('synset', 'object')
    
    self.nodes = []
    for i in xrange(10):
      self.nodes.append(g.add_node(i, [('synset', Synset())]))
        
    self.nodes[0].synset.lu_set = {LU('podatek', 2)}
    self.nodes[1].synset.lu_set = {LU('podatek', 2)}
    self.nodes[2].synset.lu_set = {LU('podatek', 2)}
    self.nodes[3].synset.lu_set = {LU('podatek', 2)}
    self.nodes[4].synset.lu_set = {LU('unia', 2)}
    self.nodes[5].synset.lu_set = {LU('unia', 2)}
    self.nodes[6].synset.lu_set = {LU('europejski', 4)}
    self.nodes[7].synset.lu_set = {LU('można', 1)}
    self.nodes[8].synset.lu_set = {LU('można', 1)}
    self.nodes[9].synset.lu_set = {LU('można', 1)}
    
    
    cfg = Config()
    cfg.parse('tests/#Data/wosedon_cosine.ini')
    cfg.resources().set_tagset('nkjp')
    cfg._context = 'Document'
    ctx = cfg.context('tests/#Data/little_ccl.xml', 'tests/#Data/little_ccl.xml').next()
    
    self.graph = g
    self.ranking = WSDRanking()
    self.ctx = ctx
  
  
  def test_set_get_lemma_ranking(self):
    rank1 = {7: 124, 34: 324}
    rank2 = {}
    rank3 = {354: 1, 324: 1}
    rank4 = {3: 43, 34: 34, 24: 53}
    
    self.ranking.set_ranking_for_lemma('aaa', 'n', rank1)
    self.ranking.set_ranking_for_lemma('bbb', 'v', rank2)
    self.ranking.set_ranking_for_lemma('ccc', 'p', rank3)
    self.assertEqual(self.ranking.get_ranking_for_lemma('bbb', 'v'), rank2)
    self.assertEqual(self.ranking.get_ranking_for_lemma('ccc', 'p'), rank3)
    self.ranking.set_ranking_for_lemma('ddd', 'n', rank4)
    
    self.assertEqual(self.ranking.get_ranking_for_lemma('aaa', 'n'), rank1)
    self.assertEqual(self.ranking.get_ranking_for_lemma('ddd', 'n'), rank4)
  
  
  def test_get_lemma_on_node_dict(self):
    lon, lon_os = self.ranking.get_lemma_on_node_dict(self.ctx, self.graph)
    
    self.assertEquals(lon, lon_os)
    self.assertEquals(len(lon), 4)
    self.assertEquals(len(lon['podatek', 'n']), 4)
    self.assertEquals(len(lon['unia', 'n']), 2)
    self.assertEquals(len(lon['europejski', 'a']), 1)
    self.assertEquals(len(lon['można', 'v']), 3)
    
    for k in lon:
      for n in lon[k]:
        self.assertIn(n, self.nodes)
  
  
  def test_get_lemma_on_node_dict_ini_sumo(self):
    self.graph.create_node_attribute('sumo', 'string')
    sumonodes = []
    for i in xrange(10):
      sumonodes.append(self.graph.add_node(10+i, [('sumo', 'adest')]))
      self.graph.add_edge(i, 10+i)
    
    lon, lon_os = self.ranking.get_lemma_on_node_dict(self.ctx, self.graph, ['sumo'])
    
    self.assertEquals(len(lon), 4)
    self.assertEquals(len(lon['podatek', 'n']), 4)
    self.assertEquals(len(lon['unia', 'n']), 2)
    self.assertEquals(len(lon['europejski', 'a']), 1)
    self.assertEquals(len(lon['można', 'v']), 3)
    for k in lon:
      for n in lon[k]:
        self.assertIn(n, sumonodes)
    
    self.assertEquals(len(lon_os), 4)
    self.assertEquals(len(lon_os['podatek', 'n']), 4)
    self.assertEquals(len(lon_os['unia', 'n']), 2)
    self.assertEquals(len(lon_os['europejski', 'a']), 1)
    self.assertEquals(len(lon_os['można', 'v']), 3)
    for k in lon_os:
      for n in lon_os[k]:
        self.assertIn(n, self.nodes)
    
  
  def test_get_lemma_on_node_dict_ini_syn_msr(self):
    self.graph.create_node_attribute('msr', 'string')
    msrnodes = []
    for i in xrange(10):
      msrnodes.append(self.graph.add_node(10+i, [('msr', 'adest')]))
      self.graph.add_edge(i, 10+i)
    
    lon, lon_os = self.ranking.get_lemma_on_node_dict(self.ctx, self.graph, ['synset', 'msr'])
    
    self.assertEquals(len(lon), 4)
    self.assertEquals(len(lon['podatek', 'n']), 8)
    self.assertEquals(len(lon['unia', 'n']), 4)
    self.assertEquals(len(lon['europejski', 'a']), 2)
    self.assertEquals(len(lon['można', 'v']), 6)
    for k in lon:
      for n in lon[k]:
        self.assertIn(n, msrnodes + self.nodes)
    
    self.assertEquals(len(lon_os), 4)
    self.assertEquals(len(lon_os['podatek', 'n']), 4)
    self.assertEquals(len(lon_os['unia', 'n']), 2)
    self.assertEquals(len(lon_os['europejski', 'a']), 1)
    self.assertEquals(len(lon_os['można', 'v']), 3)
    for k in lon_os:
      for n in lon_os[k]:
        self.assertIn(n, self.nodes)
  
  
  def test_get_lemma_on_node_dict_ini_sumo_wnd(self):
    self.graph.create_node_attribute('sumo', 'string')
    self.graph.create_node_attribute('wnd', 'string')
    
    sumonodes = []
    for i in xrange(10):
      sumonodes.append(self.graph.add_node(10+i, [('sumo', 'adest')]))
      self.graph.add_edge(i, 10+i)
      
    wndnodes = []
    for i in xrange(10):
      wndnodes.append(self.graph.add_node(20+i, [('wnd', 'adest')]))
      self.graph.add_edge(i, 20+i)
    
    lon, lon_os = self.ranking.get_lemma_on_node_dict(self.ctx, self.graph, ['sumo', 'wnd'])
    
    self.assertEquals(len(lon), 4)
    self.assertEquals(len(lon['podatek', 'n']), 8)
    self.assertEquals(len(lon['unia', 'n']), 4)
    self.assertEquals(len(lon['europejski', 'a']), 2)
    self.assertEquals(len(lon['można', 'v']), 6)
    for k in lon:
      for n in lon[k]:
        self.assertIn(n, sumonodes + wndnodes)
    
    self.assertEquals(len(lon_os), 4)
    self.assertEquals(len(lon_os['podatek', 'n']), 4)
    self.assertEquals(len(lon_os['unia', 'n']), 2)
    self.assertEquals(len(lon_os['europejski', 'a']), 1)
    self.assertEquals(len(lon_os['można', 'v']), 3)
    for k in lon_os:
      for n in lon_os[k]:
        self.assertIn(n, self.nodes)
  
  
  def test_get_context_ranking(self):
    rank = {node: -float(rank) for rank, node in enumerate(self.nodes)}
    
    self.ranking.set_ranking_for_lemma('podatek', 'n', rank)
    self.ranking.set_ranking_for_lemma('w', None, rank)
    self.ranking.set_ranking_for_lemma('unia', 'n', rank)
    self.ranking.set_ranking_for_lemma('europejski', 'a', rank)
    self.ranking.set_ranking_for_lemma('można', 'v', rank)
    
    res = self.ranking.get_ranking_for_context(self.ctx, self.graph)
    
    tks = self.ctx.tokens()
    
    def check(ord_, cnt):
      r = res[ord_]
      self.assertEquals(len(r), 2)
      self.assertIs(r[0], tks[ord_])
      if cnt == 0:
        self.assertIs(r[1], None)
      else:
        self.assertEquals(len(r[1]), cnt)
        for i in r[1]:
          self.assertEquals(len(i), 2)
          self.assertIn(i[0], self.nodes)
          self.assertIsInstance(i[1], float)
    
    self.assertEquals(len(res), 5)
    check(0, 4)
    check(1, 0)
    check(2, 2)
    check(3, 1)
    check(4, 3)
    
    






#
# <one line to give the library's name and an idea of what it does.>
# Copyright (C) 2014  <copyright holder> <email>
# 
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
# 
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# 
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
# 
#

from collections import defaultdict

from wosedon.basegraph import BaseGraph
from mergerinterface import MergerInterface

class SynsetsLUMerger2(MergerInterface):
  """!
  Class for adding edges from lexical units graph to synsets graph. 

  The purpose of this class is to add edges from lexical units graph to synsets 
  graph. New edges between synsets are added according to relation between lexical 
  units edges. Thus, each edge from lexical units graph is inserted to the synsets graph.
  """
  def __init__(self, resources, options, str_name = 'SynsetsLUMerger2'):
    """!
    @param resources - object of Resources class contain resources from config 
                       file
    @param options   - object of MergeOptions class contain merge options from 
                       config file
    @param str_name  - default value same as the class name "SynsetsLUMerger2"
    """
    super(SynsetsLUMerger2, self).__init__(resources, options, str_name)

  def merge(self, g1, g2):
    """!
    Only edges from lexical units graph are added to synsets graph. The final 
    graph contain one type of nodes, namely synsets nodes. Each synset node has 
    an attribute named "synset", to which is assigned synset object. Synset object 
    consist of: synset_id and lu_set, which contains lexical units objects. Whereas 
    lexical units objects consists of: lu_id, lemma, pos, variant and domain. The final 
    graph contain two types of relation. First type is relation occurring between synsets 
    nodes only, which have assigned relation identifier from PLWN. Second type is 
    relation occurring between lexical units, but proper synsets nodes are connected. 
    If two lexical units are connected in lexical units graph and they exist in 
    lu_set in some synsets then synsets nodes will be connected. This connection has 
    two attributes: "rel_id" and "rel_type". "rel_id" is a relation identifier 
    from lexical units graph, whereas to "rel_type" is assigned word "lu".

    The directness of the graph and uniqueness of edges are inherited from synsets graph.

    @param g1 - synsets graph
    @param g2 - lexical units graph
    @return object of BaseGraph class
    """
    g = BaseGraph()
    g.copy_graph_from(g1)
    
    lu_on_vertex_dict = defaultdict(set)
    for node in g.all_nodes():
      lu_set = node.synset.lu_set
      for lu in lu_set:
        lu_on_vertex_dict[lu.lu_id].add(node)
    
    for edge in g2.all_edges():
      parent_lu_id = edge.source().lu.lu_id
      parent_node_set = None
      if lu_on_vertex_dict.has_key(parent_lu_id):
        parent_node_set = lu_on_vertex_dict[parent_lu_id]
      else:
        continue

      child_lu_id = edge.target().lu.lu_id
      child_node_set = None
      if lu_on_vertex_dict.has_key(child_lu_id):
        child_node_set = lu_on_vertex_dict[child_lu_id]
      else:
        continue
      
      for p_node in parent_node_set:
        for ch_node in child_node_set:
          g.add_edge(p_node, 
                     ch_node, 
                     [("rel", edge.rel)], 
                     simply = True)

    return g



import sys
import argparse
try:
    import argcomplete
except ImportError:
    argcomplete = None

from npsemrel.carrot.db import db

# from plwn_graph_old import PLWNGraph
from plwn_graph_new import PLWNGraph

def make_parser():
  desc = 'PLWN graph builder...'
  parser = argparse.ArgumentParser(description = desc)
  parser.add_argument('-d', '--db-config', dest = 'db_config', required = True)
  parser.add_argument('-is', '--in-syn-graph-file', dest = 'in_syn_graph_file', required = False)
  parser.add_argument('-il', '--in-lu-graph-file', dest = 'in_lu_graph_file', required = False)
  parser.add_argument('-o', '--out-graphs-file', dest = 'out_graphs_file', required = False)
  if argcomplete:
    argcomplete.autocomplete(parser)
  return parser

def main(argv = None):
  parser = make_parser()
  args = parser.parse_args(argv)

  PLWN_G = PLWNGraph()


  if args.in_syn_graph_file:
    PLWN_G.load_syn_graph(args.in_syn_graph_file)
  
  if args.in_lu_graph_file:
    PLWN_G.load_lu_graph(args.in_lu_graph_file)
  
  if not args.in_syn_graph_file and not args.in_lu_graph_file:
    print >> sys.stderr, 'Connecting to DB...',
    dbcon = db.DB()
    dbconnection = dbcon.connect(args.db_config)
    if not dbconnection:
      print >> sys.stderr, 'Cannot connect to DB!'
      exit(1)
    print >> sys.stderr, ' Done!'

    PLWN_G.build_graphs(dbconnection)
    if args.out_graphs_file:
      PLWN_G.save_graphs(args.out_graphs_file)

  '''
  if PLWN_G.syn_G:
    print 'SYNSET GRAPH:'
    for v in PLWN_G.syn_G.vertices():
      print v, 'Synset:', PLWN_G.syn_G.vertex_properties["synset"][v].synset_id,
      for lu in PLWN_G.syn_G.vertex_properties["synset"][v].lu_set:
        print 'LU:', lu.lu_id, lu.lemma, lu.pos, lu.domain, lu.variant,
      print
    
    for e in PLWN_G.syn_G.edges():
      print e, PLWN_G.syn_G.edge_properties["rel_id"][e]

  if PLWN_G.lu_G:
    print 'LU GRAPH:'
    for v in PLWN_G.lu_G.vertices():
      print v, 'LU:', PLWN_G.lu_G.vertex_properties["lu"][v].lu_id, \
                      PLWN_G.lu_G.vertex_properties["lu"][v].lemma, \
                      PLWN_G.lu_G.vertex_properties["lu"][v].pos, \
                      PLWN_G.lu_G.vertex_properties["lu"][v].domain, \
                      PLWN_G.lu_G.vertex_properties["lu"][v].variant
    
    for e in PLWN_G.lu_G.edges():
      print e, PLWN_G.lu_G.edge_properties["rel_id"][e]
  '''


if __name__ == '__main__':
    main()



#
# <one line to give the library's name and an idea of what it does.>
# Copyright (C) 2014  <copyright holder> <email>
# 
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
# 
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# 
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
# 
#

import os
from wosedon.basegraph import BaseGraph
from graphbuilderinterface import GraphBuilderInterface

class SUMOGraphBuilder(GraphBuilderInterface):
  """!
  Class for building SUMO graph. 

  The built graph is based on Suggested Upper Merged Ontology (SUMO) graph file. 
  The Suggested Upper Merged Ontology (SUMO) is a formal upper and medium level 
  ontology including definitions of concepts and selected instance organised into 
  a network based on a few ontological relations:
    - subclass,
    - subrelation,
    - instance,
    - subAttribute.

  Format of SUMO graph file is presented below. First collumn in this file represent 
  parent name, second collumn represent child name, while last column represent 
  relation type between them.

  Format of SUMO graph file:
  \code
  parent;child;relation_type
  \endcode
  
  Sample format of SUMO graph file:
  \code
  AGM;AGM114;subclass
  AGM;AGM65;subclass
  WeatherFront;StationaryFront;subclass
  WeatherFront;WarmFront;subclass
  WeatherFront;OccludedFront;subclass
  WeatherFront;ColdFront;subclass
  ...
  \endcode
  
  SUMO ontology could be download from \link http://www.adampease.org/OP/ \endlink
  """
  def __init__(self, resources, options, str_name = 'SUMOGraphBuilder'):
    """!
    @param resources - object of Resources class contain resources from config 
                       file
    @param options   - object of BuildOptions class contain build options from 
                       config file
    @param str_name  - default value same as the class name 
                       "SUMOGraphBuilder"
    """
    super(SUMOGraphBuilder, self).__init__(resources, options, str_name)
  
  def build_graph(self):
    """!
    Build SUMO graph. 

    Build graph based on SUMO graph file. Path to this file is set in config file. 
    Each node has an attribute named "sumo" which contain concept name, e.g. 
    WeatherFront. Whereas to each edge is assigned the relation type which occurs 
    between linked SUMO concepts.

    The directness of the graph and uniqueness of edges is set in the config file. 

    @return object of BaseGraph class
    """
    if not os.path.exists(self.resources().sumo_graph_file()):
      raise IOError(
        "%s file not found!" % \
          self.resources().sumo_graph_file())

    g = BaseGraph()
    g.init_graph(drctd = self.options().is_directed_graph())

    g.create_node_attribute("sumo", "string")
    g.create_edge_attribute("rel", "string")

    with open(self.resources().sumo_graph_file()) as sgfile:
      for line in sgfile:
        parent, child, rel = line.strip().split(';')[0:3]

        g.add_node(parent, [("sumo", parent)])
        g.add_node(child, [("sumo", child)])
        g.add_edge(parent, child, [("rel", rel)])
    
    if self.options().has_unique_edges():
      g.remove_edge_duplicates()

    return g
